# flake8: noqa

from .input_types import *
from .exceptions import ObjectNotExistException, MutationValidationException
from .mutation_by_filter import (
    mutation_on_uuids_from_filter,
    mutation_on_uuids_from_filter_business_model,
)



import json
import os
import logging
import sys
import uuid

from datetime import datetime as py_datetime
import datetime as base_datetime
from cached_property import cached_property
from django.db import models
from django.db.models import Q, JSONField

# from core.datetimes.ad_datetime import datetime as py_datetime


logger = logging.getLogger(__name__)


class ExtendableModel(models.Model):
    json_ext = JSONField(db_column="JsonExt", blank=True, null=True)

    class Meta:
        abstract = True


class UUIDModel(models.Model):
    """
    Abstract entity, parent of all (new) openIMIS entities.
    Enforces the UUID identifier.
    """

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta:
        abstract = True

    def __str__(self):
        return "[%s]" % (self.id,)


class ModuleConfiguration(UUIDModel):
    """
    Generic entity to save every modules' configuration (json format)
    """

    module = models.CharField(max_length=80)
    MODULE_LAYERS = [("fe", "frontend"), ("be", "backend")]
    layer = models.CharField(
        max_length=2,
        choices=MODULE_LAYERS,
        default="be",
    )
    version = models.CharField(max_length=10)
    config = models.TextField()
    # is_exposed indicates wherever a configuration is safe to be accessible from api
    # DON'T EXPOSE (backend) configurations that contain credentials,...
    is_exposed = models.BooleanField(default=False)
    is_disabled_until = models.DateTimeField(default=None, blank=True, null=True)

    @classmethod
    def get_or_default(cls, module, default, layer="be"):
        if bool(os.environ.get("NO_DATABASE", False)):
            logger.info(
                "env NO_DATABASE set to True: ModuleConfiguration not loaded from db!"
            )
            return default

        try:
            now = py_datetime.now()  # can't use core config here...
            qs = cls.objects.filter(
                Q(is_disabled_until=None) | Q(is_disabled_until__lt=now),
                layer=layer,
                module=module,
            ).first()
            if qs:
                db_configuration = qs._cfg
                return {**default, **db_configuration}
            else:
                logger.info("No %s configuration, using default!" % module)
                return default
        except Exception:
            logger.error(
                "Failed to load %s configuration, using default!\n%s: %s"
                % (module, sys.exc_info()[0].__name__, sys.exc_info()[1])
            )
            return default

    @cached_property
    def _cfg(self):
        import collections

        return json.loads(self.config, object_pairs_hook=collections.OrderedDict)

    def __str__(self):
        return "%s [%s]" % (self.module, self.version)

    class Meta:
        db_table = "core_ModuleConfiguration"


class FieldControl(UUIDModel):
    module = models.ForeignKey(
        ModuleConfiguration, models.DO_NOTHING, related_name="controls"
    )
    field = models.CharField(unique=True, max_length=250)
    # mask: Hidden | Readonly | Mandatory
    usage = models.IntegerField(blank=True, null=True)

    class Meta:
        db_table = "core_FieldControl"


class Language(models.Model):
    code = models.CharField(db_column="LanguageCode", primary_key=True, max_length=5)
    name = models.CharField(db_column="LanguageName", max_length=50)
    sort_order = models.IntegerField(db_column="SortOrder", blank=True, null=True)

    class Meta:
        managed = True
        db_table = "tblLanguages"


def resolved_id_reference(instance, data):
    return resolve_id_reference(instance, data)


def resolve_id_reference(instance, data):
    out = {}
    for k, v in data.items():
        found = False
        if k.endswith("_id") and v:
            if hasattr(instance, k[:-3]):
                field = instance._meta.get_field(k[:-3])
                try:
                    out[k[:-3]] = field.remote_field.model.objects.get(pk=v)
                    found = True
                except Exception as e:
                    logger.debug(e)
                    raise Exception(
                        f"{instance.__name__}: relationship from {field.remote_field.model.__name__} : {v} not found "
                    )
        if not found:
            field = instance._meta.get_field(k)
            if isinstance(field, (base_datetime.date, base_datetime.datetime)):
                v = py_datetime.strptime(v, "%Y-%m-%d")
            out[k] = v
    return out



# flake8: noqa

from core.services.utils.serviceUtils import (
    check_authentication,
    model_representation,
    output_exception,
    output_result_success,
    build_delete_instance_payload,
    get_generic_type,
)



# Generated by Django 2.1.11 on 2019-08-30 16:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0003_control_mutationlog"),
    ]

    operations = [
        migrations.AddField(
            model_name="mutationlog",
            name="client_mutation_label",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="mutationlog",
            name="status",
            field=models.IntegerField(
                choices=[(0, "Received"), (1, "Error"), (2, "Success")], default=0
            ),
        ),
    ]



# Generated by Django 3.2.19 on 2023-05-16 17:42

import core.fields
import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0021_set_managed_to_true"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="legacy_id",
            field=models.UUIDField(blank=True, db_column="LegacyID", null=True),
        ),
        migrations.AddField(
            model_name="user",
            name="validity_from",
            field=core.fields.DateTimeField(
                db_column="ValidityFrom", default=datetime.datetime.now
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="validity_to",
            field=core.fields.DateTimeField(
                blank=True, db_column="ValidityTo", null=True
            ),
        ),
    ]



from nepalicalendar import NepCal
from nepalicalendar import values
from ..datetimes.ne_datetime import date
from ..datetimes.shared import datetimedelta

"""
Nepali calendar (from https://github.com/nepalicalendar/nepalicalendar-py),
wrapped with openIMIS data handling helpers
"""


def weekday(year, month, day):
    return NepCal.weekday(year, month, day)


def monthrange(year, month):
    return (NepCal.weekday(year, month, 1), NepCal.monthrange(year, month))


def weekfirstday(dt: date):
    dt.update()
    return (dt - datetimedelta(days=dt.weekday())).update()


def weeklastday(dt: date):
    dt.update()
    return (dt + datetimedelta(days=(6 - dt.weekday()))).update()


def monthfirstday(year, month):
    return date(year, month, 1).update()


def monthlastday(year, month):
    return date(year, month, monthrange(year, month)[1]).update()


def yearfirstday(year):
    return date(year, 1, 1).update()


def yearlastday(year):
    return (yearfirstday(year) + datetimedelta(days=yeardayscount(year) - 1)).update()


def monthdayscount(year: int, month: int):
    return monthrange(year, month)[1]


def yearmonthscount(year: int):
    return 12


def yeardayscount(year: int):
    return sum(values.NEPALI_MONTH_DAY_DATA[year])



# Generated by Django 3.2.16 on 2022-11-27 11:49

import core.fields
import core.models
import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0016_add_last_login_on_interactive_user"),
    ]

    operations = [
        migrations.CreateModel(
            name="ExportableQueryModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("model", models.CharField(max_length=255)),
                ("content", models.FileField(upload_to=core.models._query_export_path)),
                ("sql_query", models.TextField()),
                (
                    "create_date",
                    core.fields.DateTimeField(
                        db_column="DateCreated", default=datetime.datetime.now
                    ),
                ),
                (
                    "expire_date",
                    core.fields.DateTimeField(
                        db_column="DateExpiring",
                        default=core.models._get_default_expire_date,
                    ),
                ),
                ("is_deleted", models.BooleanField(default=False)),
                (
                    "user",
                    models.ForeignKey(
                        db_column="User",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="data_exports",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from django.db.models import Lookup, JSONField
from django.db.models.lookups import Contains
from itertools import chain

from django.conf import settings


class JsonContains(Lookup):
    lookup_name = "contains"

    sql_server_json_key_prefix = "$"
    sql_server_nested_json_separator = "."
    BASE_SQL = "(JSON_VALUE(cast({} as nvarchar(max)), %s)=%s)"

    def as_sql(self, compiler, connection):
        lhs, lhs_params = self.process_lhs(compiler, connection)
        rhs, rhs_params = self.process_rhs(compiler, connection)
        # have the json section and value of the json section in params
        params = list(
            chain.from_iterable(
                [
                    [
                        f"'{list(rhs_params[0].keys())[_]}'",
                        f"'{rhs_params[0][list(rhs_params[0].keys())[_]][0]}'",
                    ]
                    for _ in range(0, len(rhs_params[0]))
                ]
            )
        )
        sql_statement = " AND ".join(
            [self.BASE_SQL.format(lhs) for _ in range(0, len(rhs_params[0]))]
        )
        return sql_statement, params

    def _prepare_dict_value(self, json_ext):
        def flatten(dictionary, separator, prefix):
            return (
                {
                    prefix + separator + key if prefix else key: value
                    for nested_key, nested_value in dictionary.items()
                    for key, value in flatten(
                        nested_value, separator, nested_key
                    ).items()
                }
                if isinstance(dictionary, dict)
                else {prefix: str(dictionary)}
            )

        base_prefix = self.sql_server_json_key_prefix
        base_separator = self.sql_server_nested_json_separator
        return flatten(json_ext, base_separator, base_prefix)

    def _build_sql_params(
        self,
        entity,
        json_conditions,
    ):
        adjusted_conditions = self._prepare_dict_value(json_conditions)
        conditions = []
        for json_key, expected_value in adjusted_conditions.items():
            # conditions.append(entity)
            conditions.append(json_key)
            conditions.append(expected_value)
        return conditions


class JsonContainsKey(Contains):
    lookup_name = "containskey"

    def __init__(self, lhs, rhs):
        rhs = f'"{rhs}":'
        super().__init__(lhs, rhs)

    def get_rhs_op(self, connection, rhs):
        return connection.operators["contains"] % rhs


if settings.MSSQL:
    JSONField.register_lookup(JsonContains)
    JSONField.register_lookup(JsonContainsKey)



from django.test import TestCase
from django.urls import reverse
from rest_framework.test import APIClient
from rest_framework import status
from core.test_helpers import create_test_interactive_user


class CurrentUserAPITestCase(TestCase):
    def setUp(self):
        self.client = APIClient()
        self.url = reverse(
            "user-current-user"
        )  # This matches the DRF default naming for actions

        # Create a test user
        self.user = create_test_interactive_user()

    def test_get_current_user(self):
        # Authenticate the user
        self.client.force_authenticate(user=self.user)

        # Make the GET request
        response = self.client.get(self.url)

        # Check status code
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        # Check the structure of the response
        data = response.json()

        # Check main user fields
        self.assertIn("id", data)
        self.assertIsInstance(data["id"], str)
        self.assertIn("username", data)
        self.assertIsInstance(data["username"], str)

        # Check i_user fields
        self.assertIn("i_user", data)
        i_user = data["i_user"]
        self.assertIsInstance(i_user, dict)

        expected_i_user_fields = [
            "id",
            "language",
            "last_name",
            "other_names",
            "health_facility_id",
            "rights",
            "has_password",
        ]
        for field in expected_i_user_fields:
            self.assertIn(field, i_user)

        # Check specific field types
        self.assertIsInstance(i_user["id"], int)
        self.assertIsInstance(i_user["language"], str)
        self.assertIsInstance(i_user["last_name"], str)
        self.assertIsInstance(i_user["other_names"], str)
        self.assertIsInstance(i_user["rights"], list)
        self.assertIsInstance(i_user["has_password"], bool)

        # Check t_user field
        self.assertIn("t_user", data)
        self.assertIsNone(data["t_user"])  # Assuming t_user is always null in this case

    def test_unauthenticated_access(self):
        # Test without authentication
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)



import logging


logger = logging.getLogger(__name__)


class MaskingClassRegistryPoint:

    REGISTERED_MASKING_CLASS = []

    @classmethod
    def register_masking_class(cls, masking_class_list) -> None:
        for masking_class in masking_class_list:
            cls.__collect_mask_class(masking_class)

    @classmethod
    def __collect_mask_class(cls, masking_class) -> None:
        cls.REGISTERED_MASKING_CLASS.append(
            {
                "class_reference": masking_class,
                "name": masking_class.__class__.__name__,
                "model": masking_class.masking_model,
            }
        )



from rest_framework.test import APITestCase, APIClient
from rest_framework import status
from core.test_helpers import create_test_interactive_user
from core.models.user import User

from dataclasses import dataclass
from graphql_jwt.shortcuts import get_token


@dataclass
class DummyContext:
    """ Just because we need a context to generate. """
    user: User


class CurrentUserAPITest(APITestCase):
    admin_token = None
    admin_user = None
    admin_password = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.client = APIClient()
        cls.admin_password = "testCUAdmin?1"
        cls.admin_user = create_test_interactive_user(username="testCUAdmin", password=cls.admin_password)
        cls.admin_token = get_token(cls.admin_user, DummyContext(user=cls.admin_user))

    def test_authenticated_get_current_user(self):
        url = '/api/core/users/current_user/'
        # Authenticate the client using JWT token
        self.admin_user._u.given_name = "my test"
        self.admin_user._u.save_history()
        self.admin_user._u.save()
        self.client.credentials(HTTP_AUTHORIZATION=f'Bearer {self.admin_token}')
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        # Assert response contains user data, e.g., username or id
        self.assertEqual(response.data['username'], self.admin_user.username)

    def test_unauthenticated_get_current_user(self):
        url = '/api/core/users/current_user/'
        # No authentication
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)  # Or 403, depending on permissions



# Generated by Django 3.2.18 on 2023-05-15 13:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0020_add_missing_fields_to_django_scheme"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="interactiveuser",
            options={"managed": True},
        ),
        migrations.AlterModelOptions(
            name="language",
            options={"managed": True},
        ),
        migrations.AlterModelOptions(
            name="officer",
            options={"managed": True},
        ),
        migrations.AlterModelOptions(
            name="role",
            options={"managed": True},
        ),
        migrations.AlterModelOptions(
            name="roleright",
            options={"managed": True},
        ),
        migrations.AlterModelOptions(
            name="usergroup",
            options={"managed": True},
        ),
        migrations.AlterModelOptions(
            name="userrole",
            options={"managed": True},
        ),
    ]



import unittest
from core.custom_filters import CustomFilterWizardInterface, CustomFilterRegistryPoint


class DummyCustomFilterA(CustomFilterWizardInterface):
    pass


class DummyCustomFilterB(CustomFilterWizardInterface):
    pass


class TestCustomFilterRegistryPoint(unittest.TestCase):

    def setUp(self):
        # Clear registry before each test
        CustomFilterRegistryPoint.REGISTERED_CUSTOM_FILTER_WIZARDS.clear()

    def test_register_single_filter(self):
        CustomFilterRegistryPoint.register_custom_filters(
            "test_module", [DummyCustomFilterA]
        )

        registered = CustomFilterRegistryPoint.REGISTERED_CUSTOM_FILTER_WIZARDS
        self.assertIn("test_module", registered)
        self.assertEqual(len(registered["test_module"]), 1)
        self.assertEqual(
            registered["test_module"][0]["class_reference"], DummyCustomFilterA
        )

    def test_register_multiple_filters(self):
        CustomFilterRegistryPoint.register_custom_filters(
            "test_module", [DummyCustomFilterA, DummyCustomFilterB]
        )

        registered = CustomFilterRegistryPoint.REGISTERED_CUSTOM_FILTER_WIZARDS[
            "test_module"
        ]
        self.assertEqual(len(registered), 2)
        self.assertCountEqual(
            [entry["class_reference"] for entry in registered],
            [DummyCustomFilterA, DummyCustomFilterB],
        )

    def test_register_overwrites_duplicate_class(self):
        # First registration
        CustomFilterRegistryPoint.register_custom_filters(
            "test_module", [DummyCustomFilterA]
        )
        # Simulate re-registering DummyCustomFilterA (e.g., on reload)
        CustomFilterRegistryPoint.register_custom_filters(
            "test_module", [DummyCustomFilterA]
        )

        registered = CustomFilterRegistryPoint.REGISTERED_CUSTOM_FILTER_WIZARDS[
            "test_module"
        ]
        self.assertEqual(len(registered), 1)
        self.assertEqual(registered[0]["class_reference"], DummyCustomFilterA)

    def test_register_same_class_in_different_modules(self):
        CustomFilterRegistryPoint.register_custom_filters(
            "module_a", [DummyCustomFilterA]
        )
        CustomFilterRegistryPoint.register_custom_filters(
            "module_b", [DummyCustomFilterA]
        )

        registered = CustomFilterRegistryPoint.REGISTERED_CUSTOM_FILTER_WIZARDS
        self.assertIn("module_a", registered)
        self.assertIn("module_b", registered)
        self.assertEqual(len(registered["module_a"]), 1)
        self.assertEqual(len(registered["module_b"]), 1)
        self.assertEqual(registered["module_a"][0]["module"], "module_a")
        self.assertEqual(registered["module_b"][0]["module"], "module_b")



from abc import ABC
from typing import Type
from core.models import HistoryModel, VersionedModel
from django.dispatch import receiver
from django.db.models.signals import pre_save
from django.db import models
from datetime import datetime as py_datetime
import logging
from django.core.exceptions import ValidationError
from django.conf import settings


logger = logging.getLogger(__name__)


class BaseModelValidation(ABC):
    """
    Base validation class, by default all operations are unconditionally allowed.
    Validation methods should raise ValidationError in case of any data inconsistencies.
    """

    @property
    def OBJECT_TYPE(self) -> Type[HistoryModel]:
        """
        Django ORM model. It's expected that it'll be inheriting from HistoryModel.
        """
        raise NotImplementedError("Class has to define OBJECT_TYPE for service.")

    @classmethod
    def validate_create(cls, user, **data):
        pass

    @classmethod
    def validate_update(cls, user, **data):
        pass

    @classmethod
    def validate_delete(cls, user, **data):
        pass


# enforce object validation on every save
@receiver(pre_save)
def validator(sender, instance, **kwargs):
    if issubclass(sender, (HistoryModel, VersionedModel)):
        try:
            for f in instance._meta.get_fields():
                attr = (
                    getattr(instance, f.name)
                    if not f.one_to_many
                    and hasattr(instance, f.name)
                    and not f.many_to_many
                    else None
                )
                if (
                    hasattr(f, "default")
                    and not f.default == models.fields.NOT_PROVIDED
                    and not attr
                ):
                    setattr(
                        instance,
                        f.name,
                        f.default() if callable(f.default) else f.default,
                    )
                elif attr:
                    if isinstance(f, models.DecimalField) and f.decimal_places:
                        setattr(
                            instance,
                            f.name,
                            f"{{:.{f.decimal_places}f}}".format(float(attr)),
                        )
                    elif isinstance(f, models.IntegerField) and isinstance(attr, str):
                        setattr(instance, f.name, int(attr))
                    elif isinstance(f, models.DateField) and isinstance(attr, str):
                        setattr(
                            instance,
                            f.name,
                            py_datetime.strptime(attr[:10], "%Y-%m-%d"),
                        )
                    elif isinstance(f, models.DateTimeField) and not isinstance(
                        attr, py_datetime
                    ):
                        if hasattr(attr, "to_ad_datetime"):
                            setattr(instance, f.name, attr.to_ad_datetime())
                        elif isinstance(f, models.DateTimeField) and isinstance(
                            attr, str
                        ):
                            setattr(
                                instance,
                                f.name,
                                py_datetime.strptime(attr, "%Y-%m-%dT%H:%M:%S"),
                            )

            instance.full_clean(validate_unique=False)
        except Exception as e:
            msg = f"Object {instance.__class__.__name__} is not respecting the mandatory fields: {e}"
            if settings.IS_TESTING:
                logger.error(msg)
            else:
                raise ValidationError(msg)



# Generated by Django 3.2.16 on 2023-03-18 15:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0017_exportablequerymodel"),
    ]

    operations = [
        migrations.AlterField(
            model_name="technicaluser",
            name="username",
            field=models.CharField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="user",
            name="username",
            field=models.CharField(max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="officer",
            name="code",
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name="officer",
            name="veo_code",
            field=models.CharField(max_length=50, blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="interactiveuser",
            name="login_name",
            field=models.CharField(max_length=50),
        ),
    ]



# Generated by Django 3.0.3 on 2021-03-24 15:28
import django
from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0010_rolemutation"),
    ]

    operations = [
        migrations.DeleteModel(name="RoleMutation"),
        migrations.CreateModel(
            name="RoleMutation",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "mutation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="roles",
                        to="core.MutationLog",
                    ),
                ),
                (
                    "role",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="mutations",
                        to="core.Role",
                    ),
                ),
            ],
            options={
                "db_table": "core_RoleMutation",
                "managed": True,
            },
        ),
    ]



# flake8: noqa

from core.services.base import BaseService, wait_for_mutation
from core.services.userServices import (
    create_or_update_interactive_user,
    create_or_update_user_roles,
    create_or_update_user_districts,
    create_or_update_officer_villages,
    create_or_update_officer,
    create_or_update_claim_admin,
    create_or_update_core_user,
    change_user_password,
    set_user_password,
    reset_user_password,
    user_authentication,
)



# Generated by Django 2.1.11 on 2019-10-02 19:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0004_auto_20190830_1625"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserGroup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "db_table": "core_User_groups",
                "managed": False,
            },
        ),
    ]



import calendar
from ..datetimes.ad_datetime import date
import datetime as py_datetime
import calendar as py_calendar

"""
Standard Gregorian calendar,
wrapped with openIMIS data handling helpers
"""

for k, v in vars(calendar).items():
    if callable(v):
        globals()[k] = v


def weekfirstday(dt: date):
    return dt - py_datetime.timedelta(days=dt.weekday())


def weeklastday(dt: date):
    return dt + py_datetime.timedelta(days=(6 - dt.weekday()))


def monthfirstday(year: int, month: int):
    return date(year, month, 1)


def monthlastday(year: int, month: int):
    return date(year, month, py_calendar.monthrange(year, month)[1])


def yearfirstday(year: int):
    return date(year, 1, 1)


def yearlastday(year: int):
    return date(year, 12, 31)


def monthdayscount(year: int, month: int):
    return py_calendar.monthrange(year, month)[1]


def yearmonthscount(year: int):
    return 12


def yeardayscount(year: int):
    return 366 if py_calendar.isleap(year) else 365



# Generated by Django 3.2.18 on 2023-05-15 13:03

import core.fields
import datetime
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("location", "0001_initial"),
        ("core", "0019_extended_field"),
    ]

    operations = [
        migrations.RenameField(
            model_name="role",
            old_name="altlanguage",
            new_name="alt_language",
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="is_associated",
            field=models.BooleanField(
                blank=True,
                db_column="IsAssociated",
                help_text="has a claim admin or enrolment officer account",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="language",
            field=models.ForeignKey(
                db_column="LanguageID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="core.language",
            ),
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="last_login",
            field=models.DateTimeField(db_column="LastLogin", null=True),
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="password",
            field=models.CharField(
                blank=True,
                db_column="StoredPassword",
                help_text="By default a SHA256 of the private key (salt) and password",
                max_length=256,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="password_validity",
            field=models.DateTimeField(
                blank=True, db_column="PasswordValidity", null=True
            ),
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="private_key",
            field=models.CharField(
                blank=True,
                db_column="PrivateKey",
                help_text="The private key is actually a password salt",
                max_length=256,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="interactiveuser",
            name="uuid",
            field=models.CharField(
                db_column="UserUUID", default=uuid.uuid4, max_length=36, unique=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="address",
            field=models.CharField(
                blank=True, db_column="permanentaddress", max_length=100, null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="audit_user_id",
            field=models.IntegerField(db_column="AuditUserID"),
        ),
        migrations.AddField(
            model_name="officer",
            name="dob",
            field=models.DateField(blank=True, db_column="DOB", null=True),
        ),
        migrations.AddField(
            model_name="officer",
            name="email",
            field=models.CharField(
                blank=True, db_column="EmailId", max_length=200, null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="has_login",
            field=models.BooleanField(blank=True, db_column="HasLogin", null=True),
        ),
        migrations.AddField(
            model_name="officer",
            name="json_ext",
            field=models.JSONField(blank=True, db_column="JsonExt", null=True),
        ),
        migrations.AddField(
            model_name="officer",
            name="legacy_id",
            field=models.IntegerField(blank=True, db_column="LegacyID", null=True),
        ),
        migrations.AddField(
            model_name="officer",
            name="location",
            field=models.ForeignKey(
                blank=True,
                db_column="LocationId",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="location.Location",
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="phone",
            field=models.CharField(
                blank=True, db_column="Phone", max_length=50, null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="phone_communication",
            field=models.BooleanField(
                blank=True, db_column="PhoneCommunication", null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="substitution_officer",
            field=models.ForeignKey(
                blank=True,
                db_column="OfficerIDSubst",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="core.officer",
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="validity_from",
            field=core.fields.DateTimeField(
                db_column="ValidityFrom", default=datetime.datetime.now
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="validity_to",
            field=core.fields.DateTimeField(
                blank=True, db_column="ValidityTo", null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="veo_dob",
            field=models.DateField(blank=True, db_column="VEODOB", null=True),
        ),
        migrations.AddField(
            model_name="officer",
            name="veo_last_name",
            field=models.CharField(
                blank=True, db_column="VEOLastName", max_length=100, null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="veo_other_names",
            field=models.CharField(
                blank=True, db_column="VEOOtherNames", max_length=100, null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="veo_phone",
            field=models.CharField(
                blank=True, db_column="VEOPhone", max_length=25, null=True
            ),
        ),
        migrations.AddField(
            model_name="officer",
            name="works_to",
            field=models.DateTimeField(blank=True, db_column="WorksTo", null=True),
        ),
        migrations.AddField(
            model_name="usergroup",
            name="group",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.DO_NOTHING, to="auth.group"
            ),
        ),
        migrations.AddField(
            model_name="usergroup",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.DO_NOTHING, to="core.user"
            ),
        ),
        migrations.AddField(
            model_name="userrole",
            name="role",
            field=models.ForeignKey(
                db_column="RoleID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="user_roles",
                to="core.role",
            ),
        ),
        migrations.AddField(
            model_name="userrole",
            name="user",
            field=models.ForeignKey(
                db_column="UserID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="user_roles",
                to="core.interactiveuser",
            ),
        ),
        migrations.AlterField(
            model_name="interactiveuser",
            name="login_name",
            field=models.CharField(db_column="LoginName", max_length=50),
        ),
        migrations.AlterField(
            model_name="interactiveuser",
            name="validity_from",
            field=core.fields.DateTimeField(
                db_column="ValidityFrom", default=datetime.datetime.now
            ),
        ),
        migrations.AlterField(
            model_name="language",
            name="code",
            field=models.CharField(
                db_column="LanguageCode",
                max_length=5,
                primary_key=True,
                serialize=False,
            ),
        ),
        migrations.AlterField(
            model_name="officer",
            name="code",
            field=models.CharField(db_column="Code", max_length=50),
        ),
        migrations.AlterField(
            model_name="officer",
            name="veo_code",
            field=models.CharField(
                blank=True, db_column="VEOCode", max_length=50, null=True
            ),
        ),
        migrations.AlterField(
            model_name="role",
            name="uuid",
            field=models.CharField(
                db_column="RoleUUID", default=uuid.uuid4, max_length=36, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="role",
            name="validity_from",
            field=core.fields.DateTimeField(
                db_column="ValidityFrom", default=datetime.datetime.now
            ),
        ),
        migrations.AlterField(
            model_name="role",
            name="validity_to",
            field=core.fields.DateTimeField(
                blank=True, db_column="ValidityTo", null=True
            ),
        ),
        migrations.AlterField(
            model_name="roleright",
            name="validity_from",
            field=core.fields.DateTimeField(
                db_column="ValidityFrom", default=datetime.datetime.now
            ),
        ),
        migrations.AlterField(
            model_name="roleright",
            name="validity_to",
            field=core.fields.DateTimeField(
                blank=True, db_column="ValidityTo", null=True
            ),
        ),
        migrations.AlterField(
            model_name="userrole",
            name="validity_from",
            field=core.fields.DateTimeField(
                db_column="ValidityFrom", default=datetime.datetime.now
            ),
        ),
        migrations.AlterField(
            model_name="userrole",
            name="validity_to",
            field=core.fields.DateTimeField(
                blank=True, db_column="ValidityTo", null=True
            ),
        ),
        migrations.AlterUniqueTogether(
            name="usergroup",
            unique_together={("user", "group")},
        ),
        migrations.AlterModelTable(
            name="usergroup",
            table="core_User_groups",
        ),
    ]



# Generated by Django 2.1.11 on 2019-10-25 12:00

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0007_auto_20191008_0923"),
    ]

    operations = [
        migrations.CreateModel(
            name="Officer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        db_column="OfficerID", primary_key=True, serialize=False
                    ),
                ),
                (
                    "uuid",
                    models.CharField(
                        db_column="OfficerUUID",
                        default=uuid.uuid4,
                        max_length=36,
                        unique=True,
                    ),
                ),
                ("code", models.CharField(db_column="Code", max_length=8)),
                ("last_name", models.CharField(db_column="LastName", max_length=100)),
                (
                    "other_names",
                    models.CharField(db_column="OtherNames", max_length=100),
                ),
            ],
            options={
                "db_table": "tblOfficer",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="Role",
            fields=[
                (
                    "id",
                    models.AutoField(
                        db_column="RoleID", primary_key=True, serialize=False
                    ),
                ),
                ("uuid", models.CharField(db_column="RoleUUID", max_length=36)),
                ("name", models.CharField(db_column="RoleName", max_length=50)),
                (
                    "altlanguage",
                    models.CharField(
                        blank=True, db_column="AltLanguage", max_length=50, null=True
                    ),
                ),
                ("is_system", models.IntegerField(db_column="IsSystem")),
                ("is_blocked", models.BooleanField(db_column="IsBlocked")),
                ("validity_from", models.DateTimeField(db_column="ValidityFrom")),
                (
                    "validity_to",
                    models.DateTimeField(blank=True, db_column="ValidityTo", null=True),
                ),
                (
                    "audit_user_id",
                    models.IntegerField(blank=True, db_column="AuditUserID", null=True),
                ),
                (
                    "legacy_id",
                    models.IntegerField(blank=True, db_column="LegacyID", null=True),
                ),
            ],
            options={
                "db_table": "tblRole",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="RoleRight",
            fields=[
                (
                    "id",
                    models.AutoField(
                        db_column="RoleRightID", primary_key=True, serialize=False
                    ),
                ),
                ("right_id", models.IntegerField(db_column="RightID")),
                ("validity_from", models.DateTimeField(db_column="ValidityFrom")),
                (
                    "validity_to",
                    models.DateTimeField(blank=True, db_column="ValidityTo", null=True),
                ),
                (
                    "audit_user_id",
                    models.IntegerField(blank=True, db_column="AuditUserId", null=True),
                ),
                (
                    "legacy_id",
                    models.IntegerField(blank=True, db_column="LegacyID", null=True),
                ),
            ],
            options={
                "db_table": "tblRoleRight",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="UserRole",
            fields=[
                (
                    "id",
                    models.AutoField(
                        db_column="UserRoleID", primary_key=True, serialize=False
                    ),
                ),
                ("validity_from", models.DateTimeField(db_column="ValidityFrom")),
                (
                    "validity_to",
                    models.DateTimeField(blank=True, db_column="ValidityTo", null=True),
                ),
                (
                    "audit_user_id",
                    models.IntegerField(blank=True, db_column="AudituserID", null=True),
                ),
                (
                    "legacy_id",
                    models.IntegerField(blank=True, db_column="LegacyID", null=True),
                ),
            ],
            options={
                "db_table": "tblUserRole",
                "managed": False,
            },
        ),
    ]



import ast
import json
import logging
import uuid
from importlib import import_module
from typing import Any, Dict
from collections.abc import Mapping
import core
import graphene
import jsonschema
from django.db import models
from django.conf import settings
from django.core.exceptions import PermissionDenied, ValidationError, FieldDoesNotExist
from django.core.files.storage import default_storage
from django.db.models import Q, ForeignKey
from django.http import FileResponse
from django.utils.translation import gettext as _
from password_validator import PasswordValidator
from zxcvbn import zxcvbn
import datetime
from django.core.cache import caches
from functools import lru_cache


logger = logging.getLogger(__file__)


cache = caches["default"]

__all__ = [
    "TimeUtils",
    "full_class_name",
    "comparable",
    "filter_validity",
    "prefix_filterset",
    "assert_string_length",
    "PATIENT_CATEGORY_MASK_MALE",
    "PATIENT_CATEGORY_MASK_FEMALE",
    "PATIENT_CATEGORY_MASK_ADULT",
    "PATIENT_CATEGORY_MASK_MINOR",
    "patient_category_mask",
    "ExtendedConnection",
    "get_scheduler_method_ref",
    "ExtendedRelayConnection",
]


class TimeUtils(object):

    @classmethod
    def now(cls):
        return core.datetime.datetime.now()

    @classmethod
    def date(cls):
        return core.datetime.datetime.date(cls.now())


def full_class_name(o):
    module = o.__class__.__module__
    if module is None or module == str.__class__.__module__:
        return o.__class__.__name__
    return module + "." + o.__class__.__name__


def comparable(cls):
    """Class decorator providing generic comparison functionality"""

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self.__eq__(other)

    cls.__eq__ = __eq__
    cls.__ne__ = __ne__
    return cls


def filter_validity(arg="validity", prefix="", **kwargs):

    validity = kwargs.get(arg)
    if validity is None:
        return [Q(**{f"{prefix}validity_to__isnull": True})]
    elif isinstance(validity, str):
        validity = datetime.datetime.strptime(validity)
    validity = datetime.datetime(
        validity.year, validity.month, validity.day, 23, 59, 59
    )
    return [
        Q(**{f"{prefix}validity_from__lte": validity}),
        Q(**{f"{prefix}validity_to__isnull": True})
        | Q(**{f"{prefix}validity_to__gte": validity}),
    ]


def filter_validity_business_model(
    arg="dateValidFrom__Gte", arg2="dateValidTo__Lte", **kwargs
):
    date_valid_from = kwargs.get(arg)
    date_valid_to = kwargs.get(arg2)
    # default scenario
    if not date_valid_from and not date_valid_to:
        today = core.datetime.datetime.now()
        return __place_the_filters(date_start=today, date_end=None)

    # scenario - only date valid to set
    if not date_valid_from and date_valid_to:
        today = core.datetime.datetime.now()
        oldest = min([today, date_valid_to])
        return __place_the_filters(date_start=oldest, date_end=date_valid_to)

    # scenario - only date valid from
    if date_valid_from and not date_valid_to:
        return __place_the_filters(date_start=date_valid_from, date_end=None)

    # scenario - both filters set
    if date_valid_from and date_valid_to:
        return __place_the_filters(date_start=date_valid_from, date_end=date_valid_to)


def __place_the_filters(date_start, date_end):
    """funtion related to 'filter_validity_business_model'
    function so as to set up the chosen filters
    to filter the validity of the entity
    """
    if not date_end:
        return (
            Q(date_valid_from__isnull=False),
            Q(date_valid_to__isnull=True) | Q(date_valid_to__gte=date_start),
        )
    return (
        Q(date_valid_from__lte=date_end),
        Q(date_valid_to__isnull=True) | Q(date_valid_to__gte=date_start),
    )


def append_validity_filter(**kwargs):
    default_filter = kwargs.get("applyDefaultValidityFilter", False)
    date_valid_from = kwargs.get("dateValidFrom__Gte", None)
    date_valid_to = kwargs.get("dateValidTo__Lte", None)
    filters = []
    # check if we can use default filter validity
    if date_valid_from is None and date_valid_to is None:
        if default_filter:
            filters = [*filter_validity_business_model(**kwargs)]
        else:
            filters = []
    else:
        filters = [*filter_validity_business_model(**kwargs)]
    return filters


def flatten_dict(d, parent_key="", sep="."):
    """Flatten a nested dictionary."""
    items = []
    for k, v in d.items():
        new_key = f"{parent_key}{sep}{k}" if parent_key else k
        if isinstance(v, (dict, Mapping)):
            items.extend(flatten_dict(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


def filter_is_deleted(arg="is_deleted", **kwargs):
    is_deleted = kwargs.get(arg)
    if is_deleted is None:
        is_deleted = False
    return Q(is_deleted=is_deleted)


def prefix_filterset(prefix, filterset):
    if type(filterset) is dict:
        return {(prefix + k): v for k, v in filterset.items()}
    elif type(filterset) is list:
        return [(prefix + x) for x in filterset]
    else:
        return filterset


def assert_string_length(str_value, length):
    if length and len(str_value) > length:
        raise Exception(
            _("core.string.over_max_length")
            % {"value": str_value, "max_length": length}
        )


PATIENT_CATEGORY_MASK_MALE = 1
PATIENT_CATEGORY_MASK_FEMALE = 2
PATIENT_CATEGORY_MASK_ADULT = 4
PATIENT_CATEGORY_MASK_MINOR = 8


def patient_category_mask(insuree, target_date):
    if type(target_date) is str:
        from core import datetime

        # TODO: this should be nicer
        target_date = datetime.date(*[int(x) for x in target_date.split("-")])
    mask = 0
    if not insuree.gender:
        raise NotImplementedError(_("core.insuree.unknown_gender"))
    if not insuree.dob:
        raise NotImplementedError(_("core.insuree.unknown_dob"))

    if insuree.gender.code in ("M", "O"):
        mask = mask | PATIENT_CATEGORY_MASK_MALE
    else:
        mask = mask | PATIENT_CATEGORY_MASK_FEMALE

    if insuree.is_adult(target_date):
        mask = mask | PATIENT_CATEGORY_MASK_ADULT
    else:
        mask = mask | PATIENT_CATEGORY_MASK_MINOR
    return mask


class CachedManager(models.Manager):
    UNIQUE_FIELDS = {"pk", "id", "uuid"}
    CACHED_FK = {}

    def get(self, *args, **kwargs):
        """
        Overrides get() to use cache for single exact lookups on pk, id, or uuid.
        Returns a single instance or raises DoesNotExist/MultipleObjectsReturned.
        """
        if not getattr(self.model, "USE_CACHE", False):
            return super().get(*args, **kwargs)
        is_simple, key, value, field, lookup = self._is_simple_lookup(args, kwargs)

        if is_simple and lookup == "exact":
            # Try cache lookup for exact queries
            cache_result = self._handle_cache_lookup(field, value, lookup)
            if cache_result is not None:
                cached_qs = cache_result
                logger.debug(
                    "Cache hit for get() with key: %s",
                    get_cache_key(self.model, self._normalize_value(value)),
                )
                return list(cached_qs)[0]  # Use first() to get single instance

        # Fallback to default get() for non-simple queries or cache miss
        instance = super().get(*args, **kwargs)

        # Cache the instance for future lookups
        instance.update_cache()
        return instance

    def _normalize_value(self, value):
        """Normalize value for cache key."""
        if isinstance(value, uuid.UUID):
            return str(value)
        try:
            return int(value)
        except (ValueError, TypeError):
            return value

    def _is_simple_lookup(self, args, kwargs):
        """Check if query is a single exact or in lookup on unique fields."""
        if kwargs and len(kwargs) == 1:
            key = list(kwargs.keys())[0]
            field = key.split("__")[0] if "__" in key else key
            lookup = key.split("__")[-1] if "__" in key else "exact"
            return (
                field in self.UNIQUE_FIELDS and lookup in {"exact", "in"},
                key,
                kwargs.get(key),
                field,
                lookup,
            )
        elif args and len(args) == 1 and isinstance(args[0], Q):
            if len(args[0].children) == 1 and isinstance(args[0].children[0], tuple):
                field, value = args[0].children[0]
                lookup = field.split("__")[-1] if "__" in field else "exact"
                field = field.split("__")[0]
                return (
                    field in self.UNIQUE_FIELDS and lookup in {"exact", "in"},
                    field,
                    value,
                    field,
                    lookup,
                )
            else:
                logger.debug("Complex Q object detected: %s", args[0].children)
        return False, None, None, None, None

    def _instances_to_queryset(self, instances, ordered=False):
        """Convert a list of model instances to a QuerySet without hitting the database."""
        if not instances:
            return self.get_queryset().none()
        qs = self.get_queryset().filter(pk__in=[instance.pk for instance in instances])
        if ordered:
            qs = qs.order_by("pk")
            qs._result_cache = sorted(instances, key=lambda instance: instance.pk)
        else:
            qs._result_cache = list(instances)
        return qs

    # In CachedManager, update _handle_cache_lookup for 'exact' (similar changes for 'in' below)
    def _handle_cache_lookup(self, field, value, lookup):
        """Handle cache lookup for exact or in queries."""
        if lookup == "exact":
            cache_key = get_cache_key(self.model, self._normalize_value(value))
            cached_data = cache.get(cache_key)
            if cached_data:
                if isinstance(cached_data, dict):
                    # Instantiate from dict to ensure proper __init__ and dirtyfields state
                    cached_instance = self.model(**cached_data)
                    cached_instance._state.adding = (
                        False  # Mark as not new (DB-loaded simulation)
                    )
                    if hasattr(cached_instance, "_state"):
                        cached_instance._state.db = (
                            "default"  # Optional: Set DB alias if needed
                        )

                    for fk in self.CACHED_FK:
                        get_cached_foreign_key(cached_instance, fk)
                    logger.debug("Cache hit for key: %s", cache_key)
                    return self._instances_to_queryset([cached_instance], True)
                elif isinstance(cached_data, (uuid.UUID, str)):
                    return self._handle_cache_lookup("pk", cached_data, lookup)
                else:
                    logger.error("Wrong type in cache for key: %s", cache_key)
                    return None
            return None

        if lookup == "in":
            if not isinstance(value, (list, tuple, set)):
                return None
            values = [self._normalize_value(v) for v in value]
            cache_keys = [get_cache_key(self.model, v) for v in values]
            cached_results = cache.get_many(cache_keys)
            cached_instances = []
            uncached_values = []

            for v, ck in zip(values, cache_keys):
                data = cached_results.get(ck)
                if data:
                    if isinstance(data, dict):
                        instance = self.model(**data)
                        instance._state.adding = False
                        if hasattr(instance, "_state"):
                            instance._state.db = "default"
                        for fk in self.CACHED_FK:
                            get_cached_foreign_key(instance, fk)
                        cached_instances.append(instance)
                        logger.debug("Cache hit for key: %s", ck)
                    elif isinstance(data, (uuid.UUID, str)):
                        cache_result = self._handle_cache_lookup(
                            "pk", data, "exact"
                        )  # Note: Use 'exact' for recursion
                        if cache_result:
                            cached_instances.extend(cache_result._result_cache)
                            logger.debug("Cache hit for key: %s", ck)
                        else:
                            uncached_values.append(v)
                    else:
                        logger.error("Wrong type in cache for key: %s", ck)
                        uncached_values.append(v)
                else:
                    uncached_values.append(v)

            qs = self.get_queryset().none()
            if cached_instances:
                qs = self._instances_to_queryset(cached_instances)
            return qs, uncached_values, field

        return None

    def filter(self, *args, **kwargs):
        """
        Overrides filter() to use cache for single exact or in lookups on pk, id, or uuid.
        Returns a QuerySet to support chaining without unnecessary DB queries.
        """
        if getattr(self.model, "USE_CACHE", False):
            is_simple, key, value, field, lookup = self._is_simple_lookup(args, kwargs)
        else:
            is_simple = False
        if not is_simple:
            return super().filter(*args, **kwargs)

        # Try cache lookup
        cache_result = self._handle_cache_lookup(field, value, lookup)
        if cache_result is None:
            # Fallback to default filter for invalid lookups or cache miss
            return super().filter(*args, **kwargs)

        if lookup == "exact":
            cached_qs = cache_result
            if cached_qs is not None:
                return cached_qs
            # Cache miss, query DB and cache
            qs = super().filter(*args, **kwargs)
            if qs.exists():
                instance = qs.first()
                instance.update_cache()
            return qs

        # Handle in lookup
        cached_qs, uncached_values, field = cache_result
        if uncached_values:
            db_filter = {f"{field}__in": uncached_values}
            db_qs = super().filter(**db_filter)

            db_instances = list(db_qs)
            for instance in db_instances:
                instance.update_cache()
            # Combine cached and DB instances into a single QuerySet
            if cached_qs:
                all_instances = cached_qs._result_cache + db_instances
            else:
                all_instances = db_instances
            cached_qs = self._instances_to_queryset(all_instances, True)

        return cached_qs

    def get_from_cache(self, **kwargs):
        """
        Utility method to fetch instances from cache or DB using ORM-like syntax.
        Returns a QuerySet.
        """
        is_simple, key, value, field, lookup = self._is_simple_lookup((), kwargs)
        if not is_simple:
            return None

        cache_result = self._handle_cache_lookup(field, value, lookup)
        if cache_result is None:
            return None

        if lookup == "exact":
            cached_qs = cache_result
            if cached_qs is not None:
                return cached_qs
            return None

        cached_qs, uncached_values, field = cache_result
        if uncached_values:
            return None

        return cached_qs


def get_cached_foreign_key(instance, fk_field_name):
    """
    Retrieves a ForeignKey-related object from cache for a given model instance, without querying the database.
    Args:
        instance: The model instance (e.g., User instance).
        fk_field_name: The name of the ForeignKey field (e.g., 'i_user').
    Returns:
        The cached related object or None if not in cache or invalid.
    """

    logger.debug(
        "get_cached_foreign_key called: instance=%s, fk_field_name=%s",
        instance,
        fk_field_name,
    )
    # do nothing if exists already
    # if getattr(instance, fk_field_name, None) is not None:
    #     return None
    # Get the model field
    try:
        field = instance._meta.get_field(fk_field_name)
    except FieldDoesNotExist:
        logger.error(
            "Field %s does not exist on model %s",
            fk_field_name,
            instance._meta.model_name,
        )
        return None

    # Verify it's a ForeignKey
    if not isinstance(field, ForeignKey):
        logger.error(
            "Field %s on model %s is not a ForeignKey",
            fk_field_name,
            instance._meta.model_name,
        )
        return None

    # Get the ForeignKey value (e.g., i_user_id)
    fk_value = getattr(
        instance, field.attname
    )  # attname is the column name (e.g., i_user_id)
    if fk_value is None:
        logger.debug("ForeignKey value for %s is None", fk_field_name)
        return None


def clean_fk(instance):
    """
    Returns a dictionary representation of a Django model instance
    with ForeignKey fields as _id and excluding relational fields.

    Args:
        instance: A Django model instance.

    Returns:
        dict: A dictionary containing field values suitable for model instantiation.
    """
    field_values = {}
    for field in instance._meta.get_fields():
        if not isinstance(
            field,
            (
                models.ForeignKey,
                models.ManyToOneRel,
                models.ManyToManyRel,
                models.ManyToManyField,
            ),
        ):
            field_values[field.name] = getattr(instance, field.name)
        elif getattr(instance, f"{field.name}_id", None) is not None:
            field_values[f"{field.name}_id"] = getattr(instance, f"{field.name}_id")
    return field_values


class CachedModelMixin:
    USE_CACHE = settings.CACHE_OBJECT_DEFAULT

    def update_cache(self):
        """
        Updates the cache for this object after saving.
        """
        if self.USE_CACHE:
            cache.set(
                get_cache_key(self.__class__, self.pk),
                clean_fk(self),
                timeout=settings.CACHE_OBJECT_TTL,
            )
            unique_fields = getattr(
                self.__class__.objects, "UNIQUE_FIELDS", {"id", "uuid", "pk"}
            )
            for f in unique_fields:
                # get_field raised an error on property raise
                if self.pk != getattr(self, f, self.pk):
                    cache.set(
                        get_cache_key(self.__class__, getattr(self, f)),
                        self.pk,
                        timeout=settings.CACHE_OBJECT_TTL,
                    )
            logger.debug("Saved and cached instance: %s", self)

    def delete_cache(self):
        """
        Deletes the cache entry for this object.
        """
        if self.USE_CACHE:
            cache_key = f"{self.__class__.__name__}:{self.pk}"
            cache.delete(cache_key)
            logger.debug(f"Removed instance from cache: {cache_key}")


class ExtendedConnection(graphene.Connection):
    """
    Adds total_count and edge_count to Graphene connections. To use, simply add to the
    Graphene object definition Meta:
    `connection_class = ExtendedConnection`
    """

    class Meta:
        abstract = True

    total_count = graphene.Int()
    edge_count = graphene.Int()

    def resolve_total_count(self, info, **kwargs):
        if not info.context.user.is_authenticated:
            raise PermissionDenied(_("unauthorized"))
        return self.length

    def resolve_edge_count(self, info, **kwargs):
        if not info.context.user.is_authenticated:
            raise PermissionDenied(_("unauthorized"))
        return len(self.edges)


def block_update(update_dict, current_object, attribute_name, Ex=ValueError):
    if attribute_name in update_dict and update_dict["code"] != getattr(
        current_object, attribute_name
    ):
        raise Ex("That {attribute_name} field is not editable")


def get_scheduler_method_ref(name):
    """
    Use to retrieve the method reference from a str name. This is necessary when the module cannot be imported from
    that location.
    :param name: claim.module.submodule.method or similar name
    :return: reference to the method
    """
    split_name = name.split(".")
    module = __import__(".".join(split_name[:-1]))
    for subitem in split_name[1:]:
        module = getattr(module, subitem)
    return module


class ExtendedRelayConnection(graphene.relay.Connection):
    """
    Adds total_count and edge_count to Graphene Relay connections.
    """

    class Meta:
        abstract = True

    total_count = graphene.Int()
    edge_count = graphene.Int()

    def resolve_total_count(self, info, **kwargs):
        return len(self.iterable)

    def resolve_edge_count(self, info, **kwargs):
        return len(self.edges)


def get_first_or_default_language():
    from core.models import Language

    sorted_languages = Language.objects.filter(sort_order__isnull=False)
    if sorted_languages.exists():
        return sorted_languages.order_by("sort_order").first()
    else:
        return Language.objects.first()


def insert_role_right_for_system(system_role, right_id, apps):
    RoleRight = apps.get_model("core", "RoleRight")
    Role = apps.get_model("core", "Role")
    existing_roles = Role.objects.filter(
        is_system=system_role, validity_to__isnull=True
    )
    if not existing_roles:
        logger.warning(
            "Migration requested a role_right for system role %s but couldn't find that role",
            system_role,
        )
    else:
        for existing_role in existing_roles:
            role_rights = RoleRight.objects.filter(
                role=existing_role, right_id=right_id
            ).first()
            if not role_rights:
                RoleRight.objects.create(
                    role=existing_role,
                    right_id=right_id,
                    validity_from=datetime.datetime.now(),
                )


def remove_role_right_for_system(system_role, right_id, apps):
    RoleRight = apps.get_model("core", "RoleRight")
    Role = apps.get_model("core", "Role")
    existing_roles = Role.objects.filter(
        is_system=system_role, validity_to__isnull=True
    )
    if not existing_roles:
        logger.warning(
            "Migration requested to remove a role_right for system role %s but couldn't find that role",
            system_role,
        )
    for existing_role in existing_roles:
        role_rights = RoleRight.objects.filter(role=existing_role, right_id=right_id)
        if not role_rights:
            logger.warning(
                "Role right not found for system role %s and right ID %s",
                system_role,
                right_id,
            )
        for role_right in role_rights:
            role_right.delete()
            logger.info(
                "Role right removed for system role %s and right ID %s",
                system_role,
                right_id,
            )


def convert_to_python_value(string):
    try:
        value = ast.literal_eval(string)
        return value
    except (SyntaxError, ValueError):
        return string


def is_valid_uuid(string):
    try:
        uuid.UUID(str(string))
        return True
    except ValueError:
        return False


def validate_json_schema(schema):
    try:
        if not isinstance(schema, dict):
            schema = json.loads(schema)
        jsonschema.Draft7Validator.check_schema(schema)
        return []
    except jsonschema.exceptions.SchemaError as schema_error:
        return [
            {
                "message": _("core.utils.schema_validation.invalid_schema")
                % {"error": str(schema_error)}
            }
        ]
    except ValueError as json_error:
        return [
            {
                "message": _("core.utils.schema_validation.invalid_json")
                % {"error": str(json_error)}
            }
        ]


def to_json_safe_value(value):
    try:
        json.dumps(value)
        return value
    except (TypeError, ValueError):
        return str(value)


class CustomPasswordValidator:
    def __init__(self, uppercase=0, lowercase=0, digits=0, symbols=0):
        self.schema = PasswordValidator()
        self.requirements = {
            "PASSWORD_UPPERCASE": "uppercase",
            "PASSWORD_LOWERCASE": "lowercase",
            "PASSWORD_DIGITS": "digits",
            "PASSWORD_SYMBOLS": "symbols",
        }
        self.set_password_policy()

    def set_password_policy(self):
        self.schema.min(settings.PASSWORD_MIN_LENGTH)
        for setting, method in self.requirements.items():
            if getattr(settings, setting) > 0:
                getattr(self.schema.has(), method)()

    def validate(self, password, user=None):
        if not self.schema.validate(password):
            raise ValidationError(self.get_help_text())
        zxcvbn_result = zxcvbn(password)
        if zxcvbn_result["score"] < 3:
            raise ValidationError(
                "Password is too weak. Avoid common patterns and dictionary words."
            )

    def get_help_text(self):
        return (
            f"Your password must be at least {settings.PASSWORD_MIN_LENGTH} characters long, "
            f"contain at least {settings.PASSWORD_UPPERCASE} uppercase letter(s), "
            f"{settings.PASSWORD_LOWERCASE} lowercase letter(s), "
            f"{settings.PASSWORD_DIGITS} number(s), and "
            f"{settings.PASSWORD_SYMBOLS} special character(s)."
        )


class DefaultStorageFileHandler:
    def __init__(self, file_path):
        self.file_path = file_path

    def save_file(self, file):
        self.check_file_path()
        default_storage.save(self.file_path, file)
        file.seek(0)

    def remove_file(self):
        if default_storage.exists(self.file_path):
            default_storage.delete(self.file_path)

    def get_file_content(self):
        if not default_storage.exists(self.file_path):
            raise FileNotFoundError("File does not exist at the specified path.")
        with default_storage.open(self.file_path, "rb") as source:
            return source.read()

    def get_file_response_csv(self, file_name=None):
        if not default_storage.exists(self.file_path):
            raise FileNotFoundError("File does not exist at the specified path.")
        response = FileResponse(default_storage.open(self.file_path, "rb"))
        response["Content-Type"] = "text/csv"
        response["Content-Disposition"] = (
            f'attachment; filename="{file_name if file_name else "default.csv"}"'
        )
        return response

    def check_file_path(self):
        if default_storage.exists(self.file_path):
            raise FileExistsError("File already exists at the specified path.")

    @staticmethod
    def list_files(directory):
        """
        Get a list of files in the specified directory within default storage.
        """
        return default_storage.listdir(directory)


class ConfigUtilMixin:
    @classmethod
    def _load_config_fields(cls, default_cfg: Dict[str, Any]):
        """
        Load all config fields that match current AppConfig class fields, all custom fields have to be loaded separately
        """
        for field in default_cfg:
            if hasattr(cls, field):
                setattr(cls, field, default_cfg[field])

    @classmethod
    def _load_config_function(cls, function_name, path):
        """
        Load a function specified as module path into config.
        Example:
        "core.apps.function" will be loaded as "from core.apps import function" and assigned as "cls.function_name"
        """
        try:
            mod, name = path.rsplit(".", 1)
            if not mod or not name:
                raise ImportError(
                    "Invalid function path, module and function name are required"
                )
            module = import_module(mod)
            function = getattr(module, name)
            setattr(cls, function_name, function)
        except ImportError as e:
            logger.error(
                "Failed to configure function '%s' as '%s.%s': %s",
                path,
                cls.__name__,
                function_name,
                str(e),
            )


def clear_cache(instance):
    cache.delete(get_cache_key(instance.__class__, instance.pk))


def get_cache_key(model, id):
    return f"cs_{model.__name__}_{id}"


def is_this_session_superuser(session_key):
    from django.contrib.sessions.models import Session
    from django.utils.timezone import now
    from core.models import User

    try:
        session = Session.objects.get(session_key=session_key, expire_date__gte=now())
        data = session.get_decoded()
        user_id = data.get("_auth_user_id")
        if user_id:
            user = User.objects.get(id=user_id)
            if user.is_superuser:
                return True
    except Session.DoesNotExist:
        pass
    except Exception:
        pass

    return False


@lru_cache(maxsize=1)
def collect_all_gql_permissions():
    """
    Collect all GQL permission codes from Django app configs into a dict structure:
    {app: {perm_name: [perm_ids]}}.
    Scans for attributes in DEFAULT_CFG or DEFAULT_CONFIG ending with '_perms' that are lists.
    """
    excluded_apps = [
        "health_check.cache",
        "health_check",
        "health_check.db",
        "test_without_migrations",
        "rules",
        "graphene_django",
        "rest_framework",
        "health_check.storage",
        "channels",
        "graphql_jwt.refresh_token.apps.RefreshTokenConfig",
    ]
    all_apps = [
        app
        for app in settings.INSTALLED_APPS
        if not app.startswith("django") and app not in excluded_apps
    ]

    permissions_dict = {}
    for app in all_apps:
        try:
            app_module = __import__(f"{app}.apps")
            config_dict = None
            if hasattr(app_module.apps, "DEFAULT_CFG"):
                config_dict = flatten_dict(app_module.apps.DEFAULT_CFG)
            elif hasattr(app_module.apps, "DEFAULT_CONFIG"):
                config_dict = flatten_dict(app_module.apps.DEFAULT_CONFIG)

            if config_dict:
                app_perms = {}
                for key, value in config_dict.items():
                    if key.endswith("_perms") and isinstance(value, list):
                        app_perms[key] = [str(perm) for perm in value]
                if app_perms:  # Only add apps with permissions
                    permissions_dict[app] = app_perms
        except (ImportError, AttributeError):
            continue

    return permissions_dict


@lru_cache(maxsize=1)
def to_list_permissions():
    """Convert collected permissions to a set of all permission IDs."""
    permissions_dict = collect_all_gql_permissions()
    all_perms = set()
    for app_perms in permissions_dict.values():
        for perm_ids in app_perms.values():
            for perm_id in perm_ids:
                all_perms.add(int(perm_id))
    return sorted(list(all_perms))



# Generated by Django 2.1.8 on 2019-04-15 14:35

import core.fields
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("location", "0003_userdistrict"),
        ("auth", "0009_alter_user_last_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="InteractiveUser",
            fields=[
                (
                    "id",
                    models.AutoField(
                        db_column="UserID", primary_key=True, serialize=False
                    ),
                ),
                (
                    "legacy_id",
                    models.IntegerField(blank=True, db_column="LegacyID", null=True),
                ),
                ("last_name", models.CharField(db_column="LastName", max_length=100)),
                (
                    "other_names",
                    models.CharField(db_column="OtherNames", max_length=100),
                ),
                (
                    "phone",
                    models.CharField(
                        blank=True, db_column="Phone", max_length=50, null=True
                    ),
                ),
                ("login_name", models.CharField(db_column="LoginName", max_length=25)),
                ("role_id", models.IntegerField(db_column="RoleID")),
                (
                    "health_facility_id",
                    models.IntegerField(blank=True, db_column="HFID", null=True),
                ),
                ("validity_from", core.fields.DateTimeField(db_column="ValidityFrom")),
                (
                    "validity_to",
                    core.fields.DateTimeField(
                        blank=True, db_column="ValidityTo", null=True
                    ),
                ),
                ("audit_user_id", models.IntegerField(db_column="AuditUserID")),
                (
                    "email",
                    models.CharField(
                        blank=True, db_column="EmailId", max_length=200, null=True
                    ),
                ),
            ],
            options={
                "db_table": "tblUsers",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="Language",
            fields=[
                (
                    "code",
                    models.CharField(
                        db_column="LanguageCode",
                        max_length=2,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(db_column="LanguageName", max_length=50)),
                (
                    "sort_order",
                    models.IntegerField(blank=True, db_column="SortOrder", null=True),
                ),
            ],
            options={
                "db_table": "tblLanguages",
                "managed": False,
            },
        ),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("username", models.CharField(max_length=25, unique=True)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="""
                        The groups this user belongs to.
                        A user will get all permissions granted to each of their groups.
                        """,
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "i_user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.InteractiveUser",
                    ),
                ),
            ],
            options={
                "db_table": "core_User",
                "managed": True,
            },
        ),
        migrations.CreateModel(
            name="ModuleConfiguration",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("module", models.CharField(max_length=20)),
                ("version", models.CharField(max_length=10)),
                ("config", models.TextField()),
            ],
            options={
                "db_table": "core_ModuleConfiguration",
            },
        ),
        migrations.CreateModel(
            name="TechnicalUser",
            fields=[
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("username", models.CharField(max_length=150, unique=True)),
                ("email", models.EmailField(blank=True, max_length=254, null=True)),
                ("is_staff", models.BooleanField(default=False)),
                ("is_superuser", models.BooleanField(default=False)),
                ("validity_from", models.DateTimeField(blank=True, null=True)),
                ("validity_to", models.DateTimeField(blank=True, null=True)),
            ],
            options={
                "db_table": "core_TechnicalUser",
                "managed": True,
            },
        ),
        migrations.AddField(
            model_name="user",
            name="t_user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.TechnicalUser",
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="user_permissions",
            field=models.ManyToManyField(
                blank=True,
                help_text="Specific permissions for this user.",
                related_name="user_set",
                related_query_name="user",
                to="auth.Permission",
                verbose_name="user permissions",
            ),
        ),
    ]
    from django.apps import apps

    try:
        ClaimAdmin = apps.get_model("claim", "ClaimAdmin")
    except Exception:
        ClaimAdmin = None
    if not ClaimAdmin:
        import datetime

        operations += [
            migrations.CreateModel(
                name="ClaimAdmin",
                fields=[
                    (
                        "id",
                        models.AutoField(
                            db_column="ClaimAdminId", primary_key=True, serialize=False
                        ),
                    ),
                    (
                        "legacy_id",
                        models.IntegerField(
                            blank=True, db_column="LegacyID", null=True
                        ),
                    ),
                    (
                        "code",
                        models.CharField(
                            blank=True,
                            db_column="ClaimAdminCode",
                            max_length=50,
                            null=True,
                        ),
                    ),
                    (
                        "last_name",
                        models.CharField(
                            blank=True, db_column="LastName", max_length=100, null=True
                        ),
                    ),
                    (
                        "other_names",
                        models.CharField(
                            blank=True,
                            db_column="OtherNames",
                            max_length=100,
                            null=True,
                        ),
                    ),
                    ("dob", models.DateField(blank=True, db_column="DOB", null=True)),
                    (
                        "email_id",
                        models.CharField(
                            blank=True, db_column="EmailId", max_length=200, null=True
                        ),
                    ),
                    (
                        "phone",
                        models.CharField(
                            blank=True, db_column="Phone", max_length=50, null=True
                        ),
                    ),
                    (
                        "validity_from",
                        core.fields.DateTimeField(
                            db_column="ValidityFrom", default=datetime.datetime.now
                        ),
                    ),
                    (
                        "validity_to",
                        core.fields.DateTimeField(
                            blank=True, db_column="ValidityTo", null=True
                        ),
                    ),
                    (
                        "audit_user_id",
                        models.IntegerField(
                            blank=True, db_column="AuditUserId", null=True
                        ),
                    ),
                ],
                options={
                    "db_table": "tblClaimAdmin",
                    "managed": False,
                },
            ),
            migrations.AddField(
                model_name="claimadmin",
                name="has_login",
                field=models.BooleanField(blank=True, db_column="HasLogin", null=True),
            ),
            migrations.AddField(
                model_name="claimadmin",
                name="health_facility",
                field=models.ForeignKey(
                    blank=True,
                    db_column="HFId",
                    null=True,
                    on_delete=django.db.models.deletion.DO_NOTHING,
                    to="location.healthfacility",
                ),
            ),
            migrations.AddField(
                model_name="claimadmin",
                name="uuid",
                field=models.CharField(
                    db_column="ClaimAdminUUID",
                    default=uuid.uuid4,
                    max_length=36,
                    unique=True,
                ),
            ),
            migrations.AlterField(
                model_name="claimadmin",
                name="code",
                field=models.CharField(
                    blank=True, db_column="ClaimAdminCode", max_length=50, null=True
                ),
            ),
            migrations.AlterField(
                model_name="claimadmin",
                name="legacy_id",
                field=models.IntegerField(blank=True, db_column="LegacyID", null=True),
            ),
            migrations.AlterModelOptions(
                name="claimadmin",
                options={"managed": True},
            ),
        ]



import logging
from django.db import models

from .base import UUIDModel
from .user import User, Role
from .base_mutation import ObjectMutation, MutationLog

logger = logging.getLogger(__name__)


class RoleMutation(UUIDModel, ObjectMutation):
    role = models.ForeignKey(Role, models.DO_NOTHING, related_name="mutations")
    mutation = models.ForeignKey(MutationLog, models.DO_NOTHING, related_name="roles")

    class Meta:
        managed = True
        db_table = "core_RoleMutation"


class UserMutation(UUIDModel, ObjectMutation):
    core_user = models.ForeignKey(User, models.CASCADE, related_name="mutations")
    mutation = models.ForeignKey(MutationLog, models.DO_NOTHING, related_name="users")

    class Meta:
        managed = True
        db_table = "core_UserMutation"



# Generated by Django 4.2.9 on 2024-02-07 16:47

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0026_mutationlog_autogenerated_code"),
    ]

    operations = [
        migrations.AlterField(
            model_name="interactiveuser",
            name="last_login",
            field=models.DateTimeField(blank=True, db_column="LastLogin", null=True),
        ),
        migrations.AlterField(
            model_name="technicaluser",
            name="validity_from",
            field=models.DateTimeField(
                blank=True, default=datetime.datetime.now, null=True
            ),
        ),
    ]



import json
import logging
import types
from typing import Dict, List, Callable

import graphene
from graphene.types.generic import GenericScalar
from pandas import DataFrame

from core.custom_filters import CustomFilterWizardStorage
from core.models import ExportableQueryModel


logger = logging.getLogger(__file__)


class ExportableQueryMixin:
    export_patches: Dict[str, List[Callable[[DataFrame], DataFrame]]] = {}
    module_name: str
    object_type: str
    related_field: str

    @classmethod
    def get_module_name(cls) -> str:
        if not hasattr(cls, "module_name") or getattr(cls, "module_name") is None:
            error_msg = (
                "The class using `ExportableQueryMixin` must specify the `module_name` property "
                "when the customFilters argument is specified. Please override the `module_name` "
                "property in the Query schema to specify how to append custom filters for the specific object."
            )
            raise NotImplementedError(error_msg)
        return cls.module_name

    @classmethod
    def get_object_type(cls) -> str:
        if not hasattr(cls, "object_type") or getattr(cls, "object_type") is None:
            error_msg = (
                "The class using `ExportableQueryMixin` must specify the `object_type` property "
                "when the customFilters argument is specified. Please override the `object_type` "
                "property in the Query schema to specify how to append custom filters for the specific object."
            )
            raise NotImplementedError(error_msg)
        return cls.object_type

    @classmethod
    def get_related_field(cls):
        if not hasattr(cls, "related_field") or getattr(cls, "related_field") is None:
            return None
        return cls.related_field

    @classmethod
    def get_exportable_fields(cls):
        if not hasattr(cls, "exportable_fields"):
            raise NotImplementedError(
                "Class using `ExportableQueryMixin` has to provide either exportable_fields "
                "or overwrite`get_exportable_fields` to provide list of fields that can be exported."
            )
        return cls.exportable_fields

    @classmethod
    def get_patches_for_field(cls, field_type):
        if field_type not in cls.get_exportable_fields():
            raise AttributeError(f"Field {field_type} is not being exported.")
        return cls.export_patches.get(field_type)

    @classmethod
    def __init_subclass_with_meta__(cls, **meta_options):
        cls._setup_exportable_fields(**meta_options)
        return super(ExportableQueryMixin, cls).__init_subclass_with_meta__(
            **meta_options
        )

    @classmethod
    def _setup_exportable_fields(cls, **meta_options):
        for field in cls.get_exportable_fields():
            field_name = f"{field}_export"
            fld = getattr(cls, field)
            new_args = fld.args
            new_args["file_format"] = graphene.String()
            new_args["fields"] = graphene.List(of_type=graphene.String)
            new_args["fields_columns"] = GenericScalar()
            setattr(cls, field_name, graphene.Field(graphene.String, args=new_args))
            cls.create_export_function(field)

    @classmethod
    def _adjust_notation(cls, field):
        from graphene.utils.str_converters import to_snake_case

        return to_snake_case(field.replace(".", "__"))

    @classmethod
    def create_export_function(cls, field_name):
        new_function_name = f"resolve_{field_name}_export"
        default_resolve = getattr(cls, f"resolve_{field_name}", None)

        if not default_resolve:
            raise AttributeError(
                f"Query {cls} doesn't provide resolve function for {field_name}. "
                f"CSV export cannot be created"
            )

        def exporter(cls, self, info, **kwargs):
            custom_filters = kwargs.pop("customFilters", None)
            export_fields = [cls._adjust_notation(f) for f in kwargs.pop("fields")]
            fields_mapping = json.loads(kwargs.pop("fields_columns"))
            file_format = kwargs.pop("file_format", "csv")

            source_field = getattr(cls, field_name)
            filter_kwargs = {
                k: v for k, v in kwargs.items() if k in source_field.filtering_args
            }

            qs = default_resolve(None, info, **kwargs)
            qs = qs.filter(**filter_kwargs)
            qs = cls.__append_custom_filters(custom_filters, qs)
            export_file = ExportableQueryModel.create_csv_export(
                qs,
                export_fields,
                info.context.user,
                column_names=fields_mapping,
                patches=cls.get_patches_for_field(field_name),
                file_format=file_format,
            )

            return export_file.name

        setattr(cls, new_function_name, types.MethodType(exporter, cls))

    @classmethod
    def __append_custom_filters(cls, custom_filters, queryset):
        if custom_filters:
            module_name = cls.get_module_name()
            object_type = cls.get_object_type()
            related_field = cls.get_related_field()
            queryset = CustomFilterWizardStorage.build_custom_filters_queryset(
                module_name,
                object_type,
                custom_filters,
                queryset,
                relation=related_field,
            )
        return queryset



# Generated by Django 4.2.15 on 2024-09-03 10:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0029_alter_interactiveuser_role_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="exportablequerymodel",
            name="file_format",
            field=models.CharField(
                blank=True,
                choices=[("csv", "csv"), ("xlsx", "xlsx")],
                default="csv",
                max_length=255,
                null=True,
            ),
        ),
    ]



# Generated by Django 3.0.3 on 2021-03-23 12:24

import core.fields
import datetime
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0009_mutationlog_client_mutation_details"),
    ]

    operations = [
        migrations.CreateModel(
            name="RoleMutation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "validity_from",
                    core.fields.DateTimeField(
                        db_column="ValidityFrom", default=datetime.datetime.now
                    ),
                ),
                (
                    "validity_to",
                    core.fields.DateTimeField(
                        blank=True, db_column="ValidityTo", null=True
                    ),
                ),
                (
                    "legacy_id",
                    models.IntegerField(blank=True, db_column="LegacyID", null=True),
                ),
                (
                    "mutation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="roles",
                        to="core.MutationLog",
                    ),
                ),
                (
                    "role",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="mutations",
                        to="core.Role",
                    ),
                ),
            ],
            options={
                "db_table": "core_RoleMutation",
                "managed": True,
            },
        ),
    ]



# Generated by Django 2.1.11 on 2019-10-04 14:21

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0005_group_usergroup"),
    ]

    operations = [
        migrations.CreateModel(
            name="FieldControl",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("field", models.CharField(max_length=250, unique=True)),
                ("usage", models.IntegerField(blank=True, max_length=1, null=True)),
                (
                    "module",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="controls",
                        to="core.ModuleConfiguration",
                    ),
                ),
            ],
            options={
                "db_table": "core_FieldControl",
            },
        ),
    ]



# Generated by Django 3.0.3 on 2021-05-26 21:59

import core.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0011_auto_20210324_1528"),
    ]
    from django.apps import apps

    try:
        ClaimAdmin = apps.get_model("claim", "ClaimAdmin")
    except Exception:
        ClaimAdmin = None

    if ClaimAdmin:
        dependencies.append(("claim", "0012_item_service_jsonExtField"))

    operations = [
        migrations.AddField(
            model_name="user",
            name="claim_admin",
            # TOREVERT
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="claim.ClaimAdmin" if ClaimAdmin else "core.ClaimAdmin",
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="officer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="core.Officer",
            ),
        ),
        migrations.CreateModel(
            name="UserMutation",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "mutation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="users",
                        to="core.MutationLog",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="mutations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "db_table": "core_UserMutation",
                "managed": True,
            },
            bases=(models.Model, core.models.ObjectMutation),
        ),
    ]



from django.db import transaction

from core import TimeUtils
from core.schema import OpenIMISMutation
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied
from core.gql.gql_mutations import ObjectNotExistException
from django.utils.translation import gettext as _


class BaseMutation(OpenIMISMutation):

    @property
    def _mutation_class(self):
        raise NotImplementedError()

    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def async_mutate(cls, user, **data):
        try:
            cls._validate_mutation(user, **data)
            mutation_result = cls._mutate(user, **data)
            return mutation_result
        except Exception as exc:
            return [
                {
                    "message": "Failed to process {} mutation".format(
                        cls._mutation_class
                    ),
                    "detail": str(exc),
                }
            ]

    @classmethod
    def _validate_mutation(cls, user, **data):
        raise NotImplementedError()

    @classmethod
    def _mutate(cls, user, **data):
        raise NotImplementedError()


class BaseDeleteMutation(BaseMutation):

    class Input:
        pass

    @classmethod
    def async_mutate(cls, user, **data):
        output = []
        if "uuids" in data:
            for uuid in set(data["uuids"]):
                deletion_result = super(BaseDeleteMutation, cls).async_mutate(
                    user, uuid=uuid
                )
                deletion_result = [None] if deletion_result is None else ["error"]
                output += deletion_result
            return None if output == [None] * len(output) else output


class BaseReplaceMutation(BaseMutation):

    class Input:
        pass

    @classmethod
    def async_mutate(cls, user, **data):
        try:
            cls._validate_mutation(user, **data)
            mutation_result = cls._mutate(user, **data)
            return mutation_result
        except Exception as exc:
            return [
                {
                    "message": "Failed to process {} mutation".format(
                        cls._mutation_class
                    ),
                    "detail": str(exc),
                }
            ]


class BaseCreateMutationMixin:

    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _validate_mutation(cls, user, **data):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))

    @classmethod
    def _mutate(cls, user, **data):
        data["user_created"] = user.id_for_audit
        data["date_created"] = TimeUtils.now()

        if "client_mutation_id" in data:
            data.pop("client_mutation_id")
        if "client_mutation_label" in data:
            data.pop("client_mutation_label")
        cls.create_object(data)

    @classmethod
    def create_object(cls, object_data):
        obj = cls._model.objects.create(**object_data)
        obj.save()
        return obj


class BaseUpdateMutationMixin:

    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _object_not_exist_exception(cls, obj_uuid):
        raise ObjectNotExistException(cls._model, obj_uuid)

    @classmethod
    def _validate_mutation(cls, user, **data):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))

        obj_uuid = data["uuid"]
        if cls._model.objects.filter(uuid=data["uuid"]).first() is None:
            cls._object_not_exist_exception(obj_uuid=obj_uuid)

    @classmethod
    def _mutate(cls, user, **data):
        if "client_mutation_id" in data:
            data.pop("client_mutation_id")
        if "client_mutation_label" in data:
            data.pop("client_mutation_label")

        data["user_updated"] = user.id_for_audit
        data["date_updated"] = TimeUtils.now()
        updated_object = cls._model.objects.filter(uuid=data["uuid"]).first()
        [setattr(updated_object, key, data[key]) for key in data]
        cls.update_object(updated_object)

    @classmethod
    def update_object(cls, object_to_update):
        object_to_update.save_history()
        object_to_update.save()
        return object_to_update


class BaseDeleteMutationMixin:
    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _object_not_exist_exception(cls, obj_uuid):
        raise ObjectNotExistException(cls._model, obj_uuid)

    @classmethod
    def _validate_mutation(cls, user, **data):
        cls._validate_user(user)

    @classmethod
    def _validate_user(cls, user):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))

    @classmethod
    def _mutate(cls, uuid):
        object_to_delete = cls._model.objects.filter(uuid=uuid).first()

        if object_to_delete is None:
            cls._object_not_exist_exception(uuid)
        else:
            object_to_delete.delete_history()


class BaseHistoryModelCreateMutationMixin:

    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _validate_mutation(cls, user, **data):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))

    @classmethod
    def _mutate(cls, user, **data):
        if "client_mutation_id" in data:
            data.pop("client_mutation_id")
        if "client_mutation_label" in data:
            data.pop("client_mutation_label")
        cls.create_object(user=user, object_data=data)

    @classmethod
    def create_object(cls, user, object_data):
        obj = cls._model(**object_data)
        obj.save(username=user.username)
        return obj


class BaseHistoryModelUpdateMutationMixin:

    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _object_not_exist_exception(cls, obj_uuid):
        raise ObjectNotExistException(cls._model, obj_uuid)

    @classmethod
    def _validate_mutation(cls, user, **data):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))
        obj_uuid = data["id"]
        if cls._model.objects.filter(id=data["id"]).first() is None:
            cls._object_not_exist_exception(obj_uuid=obj_uuid)

    @classmethod
    def _mutate(cls, user, **data):
        if "date_valid_to" not in data:
            data["date_valid_to"] = None
        if "client_mutation_id" in data:
            data.pop("client_mutation_id")
        if "client_mutation_label" in data:
            data.pop("client_mutation_label")
        updated_object = cls._model.objects.filter(id=data["id"]).first()
        [setattr(updated_object, key, data[key]) for key in data]
        cls.update_object(user=user, object_to_update=updated_object)

    @classmethod
    def update_object(cls, user, object_to_update):
        object_to_update.save(username=user.username)
        return object_to_update


class BaseHistoryModelDeleteMutationMixin:
    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _object_not_exist_exception(cls, obj_uuid):
        raise ObjectNotExistException(cls._model, obj_uuid)

    @classmethod
    def _validate_mutation(cls, user, **data):
        cls._validate_user(user)

    @classmethod
    def _validate_user(cls, user):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))

    @classmethod
    def _mutate(cls, user, **data):
        if "client_mutation_id" in data:
            data.pop("client_mutation_id")
        if "client_mutation_label" in data:
            data.pop("client_mutation_label")

        id_ = data.get("uuid", None) or data.get("id", None)
        ids = data.get("uuids", None) or data.get("ids", None)
        if id_:
            cls.__delete_single_obj(user=user, id_=id_)
        elif ids:
            with transaction.atomic():
                for id_ in ids:
                    cls.__delete_single_obj(user, id_)

    @classmethod
    def __delete_single_obj(cls, user, id_):
        object_to_delete = cls._model.objects.filter(id=id_).first()
        if object_to_delete is None:
            cls._object_not_exist_exception(id_)
        else:
            object_to_delete.delete(username=user.username)


class BaseHistoryModelReplaceMutationMixin:
    @property
    def _model(self):
        raise NotImplementedError()

    @classmethod
    def _object_not_exist_exception(cls, obj_uuid):
        raise ObjectNotExistException(cls._model, obj_uuid)

    @classmethod
    def _validate_mutation(cls, user, **data):
        cls._validate_user(user)

    @classmethod
    def _validate_user(cls, user):
        if type(user) is AnonymousUser or not user.id:
            raise PermissionDenied(_("mutation.authentication_required"))

    @classmethod
    def _mutate(cls, user, **data):
        if "client_mutation_id" in data:
            data.pop("client_mutation_id")
        if "client_mutation_label" in data:
            data.pop("client_mutation_label")
        object_to_replace = cls._model.objects.filter(id=data["uuid"]).first()
        if object_to_replace is None:
            cls._object_not_exist_exception(data["uuid"])
        else:
            object_to_replace.replace_object(data=data, username=user.username)



import datetime
import uuid
import decimal
from django.test import TestCase
from django.db.models import Q

from core.utils import (
    full_class_name, comparable, to_json_safe_value,
    to_list_permissions, filter_validity, collect_all_gql_permissions
)
from core.datetimes.ad_datetime import AdDate, AdDatetime
from core.test_helpers import create_test_interactive_user
from core.models import User, RoleRight, UserRole, Role
from django.core.cache import caches


class ComparableTest(TestCase):
    def test_generic_eq(self):
        @comparable
        class A(object):
            def __init__(self, f):
                self.f = f

            def __eq__(self, other):
                return self.f == other.f

        @comparable
        class B(object):
            def __init__(self, f):
                self.f = f

            def __eq__(self, other):
                return self.f == other.f

        obj1 = A(f="a")
        obj2 = A(f="a")
        self.assertEquals(obj1, obj2)
        obj3 = B(f="b")
        self.assertNotEquals(obj1, obj3)
        obj4 = B(f="a")
        self.assertNotEquals(obj1, obj4)


class UtilsTestCase(TestCase):
    def test_full_class_name(self):
        self.assertEquals(full_class_name(self), "core.tests.test_utils.UtilsTestCase")

        self.assertEquals(full_class_name(1), "int")

    def test_json_serialize_value(self):
        self.assertEquals(to_json_safe_value(42), 42)
        self.assertEquals(to_json_safe_value("foo"), "foo")

        uuid_obj = uuid.uuid4()
        self.assertEquals(to_json_safe_value(uuid_obj), str(uuid_obj))

        date_obj = datetime.date(2025, 1, 1)
        self.assertEquals(to_json_safe_value(date_obj), str(date_obj))

        ad_date_obj = AdDate(2025, 1, 1)
        self.assertEquals(to_json_safe_value(ad_date_obj), str(ad_date_obj))

        ad_datetime_obj = AdDatetime(2025, 1, 1, 12, 0, 0)
        self.assertEquals(to_json_safe_value(ad_datetime_obj), str(ad_datetime_obj))

        decimal_obj = decimal.Decimal("12345.6789")
        self.assertEquals(to_json_safe_value(decimal_obj), str(decimal_obj))

    def test_is_admin_rights(self):
        role = Role.objects.filter(is_system=64, *filter_validity()).first()
        user = User.objects.filter(username="Admin", *filter_validity()).first()
        if not user:
            user = create_test_interactive_user(username="Admin", roles=[role.id])
        # removing all role but admin
        UserRole.objects.filter(
            ~Q(role__is_system=64), user=user._u, *filter_validity()
        ).delete()
        # removing all admin rights
        RoleRight.objects.filter(role__is_system=64, *filter_validity()).delete()
        rights = list(user.rights)
        rights_db = [
            rr.right_id
            for rr in RoleRight.filter_queryset()
            .filter(role__is_system=64, *filter_validity(prefix="role__"))
            .distinct()
        ]
        self.assertEquals(len(rights_db), 0, "all roleright are not removed")
        self.assertEquals(
            len(rights),
            len(to_list_permissions()),
            "rights are not equal to all right available",
        )
        self.assertNotEquals(
            len(rights_db), len(rights),
            "admin should always get all permissions regardless of their RoleRight stored in DB"
        )
    CORE_PERMS = [
        121701, 121702, 121703, 121704,
        122001, 122002, 122003, 122006, 122005, 122004,
        121501, 121502, 121503, 121504,
        121601, 121602, 121603, 121604,
        900101, 122000
    ]

    def test_collect_all_gql_permissions(self):
        perms = collect_all_gql_permissions()
        self.assertTrue('core' in perms)
        core_perms = perms['core']
        all_core_perm = []
        for core_perms in perms['core'].values():
            for core_perm in core_perms:
                all_core_perm.append(int(core_perm))
        self.assertEqual(sorted(all_core_perm), sorted(self.CORE_PERMS), "Missmatch for the expected core perms")

    def test_to_list_permissions(self):
        perm_list = to_list_permissions()
        for perm in self.CORE_PERMS:
            self.assertTrue((perm) in perm_list, f"This perm {perm} from core is not return ")

    def test_cache_invalidation(self):
        User.USE_CACHE = True
        users = list(User.objects.all())
        users_id = [user.id for user in users]
        users_0_no_cache_get = User.objects.get(id=users_id[0])
        users_0_filter = User.objects.filter(id=users_id[0]).first()
        self.assertEquals(
            users_0_no_cache_get,
            users_0_filter,
            "get and filter should retrieve the same object",
        )
        users_0_filter.username = users_0_filter.username + "T"
        users_0_filter.save()
        users_filter = list(User.objects.filter(id__in=users_id))
        caches["default"].delete(f"cd_User_{users_filter[2].id}")
        users.remove(users_0_no_cache_get)
        users_filter.remove(users_0_filter)
        users_0_filter = User.objects.filter(id=users_id[0]).first()
        self.assertNotEquals(
            users_0_no_cache_get.username,
            users_0_filter.username,
            "the object should be different, cache not invalidated properly",
        )
        self.assertNotEquals(
            users,
            users_filter,
            "should be the same list even if user_filter comes partially from cache",
        )
        caches["default"].clear()
        User.USE_CACHE = False



# Generated by Django 3.2.16 on 2024-01-24 11:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0025_mutationlog_json_ext"),
    ]

    operations = [
        migrations.AddField(
            model_name="mutationlog",
            name="autogenerated_code",
            field=models.TextField(blank=True, null=True),
        ),
    ]



# flake8: noqa

from .abstract_websocket_client import AbstractWebSocketClient
from .base_websocket_client import BaseWebSocketClient, JsonWebSocketClient
from .async_websocket_client import AsyncWebSocketClient



from django.test import TestCase
from core.models import User, TechnicalUser, InteractiveUser


class UserTestCase(TestCase):

    def test_t_user_active_status(self):
        always_valid = User(
            username="always_valid", t_user=TechnicalUser(username="always_valid")
        )
        self.assertTrue(always_valid.is_active)

        from core import datetime, datetimedelta

        not_yet_active = User(
            username="not_yet_active",
            t_user=TechnicalUser(
                username="not_yet_active",
                validity_from=datetime.datetime.now() + datetimedelta(days=1),
            ),
        )
        self.assertFalse(not_yet_active.is_active)

        not_active_anymore = User(
            username="not_active_anymore",
            t_user=TechnicalUser(
                username="not_active_anymore",
                validity_to=datetime.datetime.now() + datetimedelta(days=-1),
            ),
        )
        self.assertFalse(not_active_anymore.is_active)

    def test_i_active_status(self):
        always_valid = User(
            username="always_valid", i_user=InteractiveUser(login_name="always_valid")
        )
        self.assertTrue(always_valid.is_active)

        from core import datetime, datetimedelta

        not_yet_active = User(
            username="always_valid",
            i_user=InteractiveUser(
                login_name="not_yet_active",
                validity_from=datetime.datetime.now() + datetimedelta(days=1),
            ),
        )
        self.assertFalse(not_yet_active.is_active)

        not_active_anymore = User(
            username="always_valid",
            i_user=InteractiveUser(
                login_name="not_active_anymore",
                validity_to=datetime.datetime.now() + datetimedelta(days=-1),
            ),
        )
        self.assertFalse(not_active_anymore.is_active)



# Generated by Django 2.1.7 on 2019-08-21 21:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0002_auto_20190726_0701"),
    ]

    operations = [
        migrations.CreateModel(
            name="MutationLog",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("json_content", models.TextField()),
                ("request_date_time", models.DateTimeField(auto_now_add=True)),
                (
                    "client_mutation_id",
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                ("status", models.IntegerField()),
                ("error", models.TextField(blank=True, null=True)),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "db_table": "core_Mutation_Log",
                "managed": True,
            },
        ),
    ]



import sys
from nepalicalendar import NepDate
from nepalicalendar import values
import datetime as py_datetime

from .shared import datetimedelta

"""
Nepali date (from https://github.com/nepalicalendar/nepalicalendar-py),
wrapped with openIMIS data handling helpers
"""

core = sys.modules["core"]

tzinfo = py_datetime.tzinfo
timezone = py_datetime.timezone


class NeDate(NepDate):

    def raw_isoformat(self, *args, **kwargs):
        return "%04d-%02d-%02d" % (self.year, self.month, self.day)

    def ad_isoformat(self, *args, **kwargs):
        return self.to_ad_date().isoformat(*args, **kwargs)

    def isoformat(self):
        if core.iso_raw_date:
            return self.raw_isoformat()
        else:
            return self.ad_isoformat()

    def displayshortformat(self):
        self.update()
        return "%s/%s/%s" % (self.ne_day, self.ne_month, self.ne_year)

    def displaylongformat(self):
        self.update()
        return "%s %s %s %s" % (
            self.weekday_name(),
            self.ne_day,
            self.month_name(),
            self.ne_year,
        )

    def to_ad_date(self):
        start_date = NeDate(values.START_NP_YEAR, 1, 1)
        days = self - start_date
        ad_dt = values.START_EN_DATE + days
        return py_datetime.date(ad_dt.year, ad_dt.month, ad_dt.day)

    def to_ad_datetime(self):
        start_date = NeDate(values.START_NP_YEAR, 1, 1)
        days = self - start_date
        ad_date = values.START_EN_DATE + days
        return py_datetime.datetime(ad_date.year, ad_date.month, ad_date.day)

    @classmethod
    def from_ad_date(cls, dt):
        if dt is None:
            return None
        if isinstance(dt, py_datetime.datetime):
            dt = py_datetime.date(dt.year, dt.month, dt.day)
        if dt < values.START_EN_DATE:
            return date.min
        if dt > values.END_EN_DATE:
            return date.max
        ne_dte = NepDate.from_ad_date(dt)
        return NeDate(ne_dte.year, ne_dte.month, ne_dte.day).update()

    @classmethod
    def from_ad_datetime(cls, value):
        if value is None:
            return None
        ad_dt = py_datetime.date(value.year, value.month, value.day)
        return date.from_ad_date(ad_dt)

    def to_ne_datetime(self):
        return NeDatetime(self.year, self.month, self.day)

    def to_datetime(self):
        return self.to_ne_datetime()

    def replace(self, year=None, month=None, day=None):
        """Return a new date with new values for the specified fields."""
        if year is None:
            year = self.year
        if month is None:
            month = self.month
        if day is None:
            day = self.day
        return type(self)(year, month, day)

    @classmethod
    def today(cls):
        nep_today = NepDate.today()
        return NeDate(nep_today.year, nep_today.month, nep_today.day).update()

    @classmethod
    def _convert_op_res(cls, res):
        if isinstance(res, py_datetime.datetime):
            return NeDate.from_ad_datetime(res)
        if isinstance(res, py_datetime.date):
            return NeDate.from_ad_date(res)
        return res

    def __add__(self, other):
        if isinstance(other, datetimedelta):
            return datetimedelta.add_to_date(other, self)
        dt = super(NeDate, self).__add__(other)
        return NeDate._convert_op_res(dt)

    def __sub__(self, other):
        if isinstance(other, datetimedelta):
            return datetimedelta.add_to_date(-other, self)
        dt = super(NeDate, self).__sub__(other)
        return NeDate._convert_op_res(dt)

    def __repr__(self):
        L = [self.year, self.month, self.day]
        if L[-1] == 0:
            del L[-1]
        if L[-1] == 0:
            del L[-1]
        return "core.datetimes.ne_datetime.date(%s)" % (", ".join(map(str, L)))

    def __str__(self):
        return self.raw_isoformat()


date = NeDate
date.min = NeDate.from_ad_date(values.START_EN_DATE)
date.max = NeDate.from_ad_date(values.END_EN_DATE)
date.min_ad = values.START_EN_DATE
date.max_ad = values.END_EN_DATE
date.resolution = py_datetime.timedelta(days=1)


class NeDatetime(object):
    __slots__ = ["_date", "_time", "_fold"]

    def __init__(
        self,
        year,
        month,
        day,
        hour=0,
        minute=0,
        second=0,
        microsecond=0,
        tzinfo=None,
        *,
        fold=0
    ):
        self._date = date(year, month, day).update()
        self._time = py_datetime.time(hour, minute, second, microsecond, tzinfo)
        self._fold = fold

    @property
    def year(self):
        return self._date.year

    @property
    def month(self):
        return self._date.month

    @property
    def day(self):
        return self._date.day

    @property
    def hour(self):
        return self._time.hour

    @property
    def minute(self):
        return self._time.minute

    @property
    def second(self):
        return self._time.second

    @property
    def microsecond(self):
        return self._time.microsecond

    @property
    def tzinfo(self):
        return self._time.tzinfo

    @property
    def fold(self):
        return self._fold

    def raw_isoformat(self, *args, **kwargs):
        return "%s %s" % (
            self._date.raw_isoformat(),
            self._time.isoformat(*args, **kwargs),
        )

    def isoformat(self, *args, **kwargs):
        return "%s %s" % (self._date.isoformat(), self._time.isoformat(*args, **kwargs))

    @classmethod
    def now(cls):
        return cls.from_ad_datetime(py_datetime.datetime.now())

    @classmethod
    def from_ad_datetime(cls, dt):
        if dt is None:
            return None
        if dt < py_datetime.datetime(
            date.min_ad.year, date.min_ad.month, date.min_ad.day, tzinfo=dt.tzinfo
        ):
            return datetime.min
        if dt > py_datetime.datetime(
            date.max_ad.year,
            date.max_ad.month,
            date.max_ad.day,
            23,
            59,
            59,
            999999,
            tzinfo=dt.tzinfo,
        ):
            return datetime.max
        ne_dte = NeDate.from_ad_date(py_datetime.date(dt.year, dt.month, dt.day))
        return NeDatetime(
            ne_dte.year,
            ne_dte.month,
            ne_dte.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            dt.tzinfo,
        )

    def to_ad_datetime(self):
        start_date = NeDate(values.START_NP_YEAR, 1, 1)
        days = self._date - start_date
        ad_dt = values.START_EN_DATE + days
        return py_datetime.datetime(
            ad_dt.year,
            ad_dt.month,
            ad_dt.day,
            self._time.hour,
            self._time.minute,
            self._time.second,
            self._time.microsecond,
            self._time.tzinfo,
        )

    @classmethod
    def from_ad_date(cls, dt):
        if dt is None:
            return None
        if isinstance(dt, py_datetime.date):
            dt = py_datetime.datetime(dt.year, dt.month, dt.day)
        if dt < py_datetime.datetime(
            date.min_ad.year, date.min_ad.month, date.min_ad.day, tzinfo=dt.tzinfo
        ):
            return datetime.min
        if dt > py_datetime.datetime(
            date.max_ad.year,
            date.max_ad.month,
            date.max_ad.day,
            23,
            59,
            59,
            999999,
            tzinfo=dt.tzinfo,
        ):
            return datetime.max
        ne_dte = NeDate.from_ad_date(py_datetime.date(dt.year, dt.month, dt.day))
        return NeDatetime(ne_dte.year, ne_dte.month, ne_dte.day)

    def to_ad_date(self):
        start_date = NeDate(values.START_NP_YEAR, 1, 1)
        days = self.date() - start_date
        ad_dt = values.START_EN_DATE + days
        return py_datetime.date(ad_dt.year, ad_dt.month, ad_dt.day)

    def date(self):
        return self._date

    def __eq__(self, other):
        if isinstance(other, NeDatetime):
            return (
                self._date == other._date
                and self._time == other._time
                and self.fold == other.fold
            )
        if isinstance(other, NeDate):
            return (
                self.year == other.year
                and self.month == other.month
                and self.day == other.day
                and self.hour == 0
                and self.minute == 0
                and self.second == 0
                and self.microsecond == 0
                and self.fold == 0
            )
        return NotImplemented

    def _gt_ne_datetime(self, other):
        if self._date > other._date:
            return True
        if self._date < other._date:
            return False
        return self._time > other._time

    def _gt_ne_date(self, other):
        if self.year != other.year:
            return self.year > other.year
        if self.month != other.month:
            return self.month > other.month
        if self.day != other.day:
            return self.day > other.day
        return (
            self.hour > 0 or self.minute > 0 or self.second > 0 or self.microsecond > 0
        )

    def __gt__(self, other):
        if isinstance(other, py_datetime.datetime):
            other = NeDatetime.from_ad_datetime(other)
        if isinstance(other, NeDatetime):
            return self._gt_ne_datetime(other)
        if isinstance(other, NeDate):
            return self._gt_ne_date(other)
        return NotImplemented

    def __lt__(self, other):
        return (not self == other) and (not self > other)

    def __ge__(self, other):
        return not self < other

    def __le__(self, other):
        return not self > other

    def replace(
        self,
        year=None,
        month=None,
        day=None,
        hour=None,
        minute=None,
        second=None,
        microsecond=None,
        tzinfo=None,
    ):
        """Return a new date with new values for the specified fields."""
        if year is None:
            year = self.year
        if month is None:
            month = self.month
        if day is None:
            day = self.day
        if hour is None:
            hour = self.hour
        if minute is None:
            minute = self.minute
        if second is None:
            second = self.second
        if microsecond is None:
            microsecond = self.microsecond
        if tzinfo is None:
            tzinfo = self.tzinfo
        return type(self)(year, month, day, hour, minute, second, microsecond, tzinfo)

    @classmethod
    def _convert_op_res(cls, res):
        if isinstance(res, py_datetime.datetime):
            return NeDatetime.from_ad_datetime(res)
        return res

    def __add__(self, other):
        if isinstance(other, datetimedelta):
            return datetimedelta.add_to_datetime(other, self)
        if isinstance(other, NeDatetime):
            return NeDatetime.from_ad_datetime(
                self.to_ad_datetime().__add__(other.to_ad_datetime())
            )
        dt = self.to_ad_datetime().__add__(other)
        return NeDatetime._convert_op_res(dt)

    def __sub__(self, other):
        if isinstance(other, datetimedelta):
            return datetimedelta.add_to_date(-other, self)
        if isinstance(other, NeDatetime):
            return self.to_ad_datetime().__sub__(other.to_ad_datetime())
        dt = self.to_ad_datetime().__sub__(other)
        return NeDatetime._convert_op_res(dt)

    def __repr__(self):
        L = [
            self._date.year,
            self._date.month,
            self._date.day,
            self._time.hour,
            self._time.minute,
            self._time.second,
            self._time.microsecond,
        ]
        if L[-1] == 0:
            del L[-1]
        if L[-1] == 0:
            del L[-1]
        s = "%s.datetime(%s)" % (self.__class__.__module__, ", ".join(map(str, L)))
        if self._time.tzinfo is not None:
            assert s[-1:] == ")"
            s = s[:-1] + ", tzinfo=%r" % self.tzinfo + ")"
        if self._fold:
            assert s[-1:] == ")"
            s = s[:-1] + ", fold=1)"
        return s

    def __str__(self):
        s = "%s %s" % (self._date, self._time)
        if self._fold:
            assert s[-1:] == ")"
            s = s[:-1] + ", fold=1)"
        return s


datetime = NeDatetime
datetime.min = NeDatetime(date.min.year, date.min.month, date.min.day)
datetime.max = NeDatetime(
    date.max.year, date.max.month, date.max.day, 23, 59, 59, 999999
)
datetime.min_ad = py_datetime.datetime(
    date.min_ad.year, date.min_ad.month, date.min_ad.day
)
datetime.max_ad = py_datetime.datetime(
    date.max_ad.year, date.max_ad.month, date.max_ad.day, 23, 59, 59, 999999
)



# Generated by Django 3.2.20 on 2023-08-31 10:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0024_alter_usergroup_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="mutationlog",
            name="json_ext",
            field=models.JSONField(blank=True, db_column="JsonExt", null=True),
        ),
    ]



from core.utils import insert_role_right_for_system
from django.db import migrations, models

ROLE_RIGHTS_ID = [101201, 101101, 101001, 101105]
MANAGER_ROLE_IS_SYSTEM = 64  # By default missing rights are assigned to ClaimAdmin role


def create_role_right(apps, schema_editor):
    for right_id in ROLE_RIGHTS_ID:
        insert_role_right_for_system(256, right_id, apps)


class Migration(migrations.Migration):

    dependencies = [("core", "0014_users")]

    operations = [
        migrations.AddField(
            model_name="roleright",
            name="role",
            field=models.ForeignKey(
                db_column="RoleID",
                on_delete=models.deletion.DO_NOTHING,
                related_name="rights",
                to="core.role",
            ),
        ),
        migrations.RunPython(create_role_right),
    ]



from core import views
from django.urls import include, path
from rest_framework.routers import DefaultRouter

router = DefaultRouter()
router.register(r"users", views.UserViewSet)

urlpatterns = [
    path("", include(router.urls)),
    path("fetch_export", views.fetch_export),
    path("scheduled_jobs", views.get_scheduled_jobs),
]



import logging
from django.utils.timezone import now
from django_ratelimit.core import is_ratelimited
from rest_framework.exceptions import JsonResponse
from django.conf import settings


logger = logging.getLogger(__name__)


class DefaultAxesAttributesMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # Set default values for Django-axes attributes if they're not already set
        if not hasattr(request, "axes_ip_address"):
            request.axes_ip_address = request.META.get("REMOTE_ADDR", "")
        if not hasattr(request, "axes_user_agent"):
            request.axes_user_agent = request.META.get("HTTP_USER_AGENT", "")
        if not hasattr(request, "axes_attempt_time"):
            request.axes_attempt_time = now()

        return self.get_response(request)


class SecurityHeadersMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)

        if settings.MODE == "PROD":
            response["Strict-Transport-Security"] = (
                "max-age=63072000; includeSubDomains"
            )
            response["Content-Security-Policy"] = "default-src 'self';"
            response["X-Frame-Options"] = "DENY"
            response["X-Content-Type-Options"] = "nosniff"
            response["Referrer-Policy"] = "strict-origin-when-cross-origin"
            response["Permissions-Policy"] = "geolocation=(), microphone=()"

        return response


class GraphQLRateLimitMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        group = settings.RATELIMIT_GROUP
        key = settings.RATELIMIT_KEY
        rate = settings.RATELIMIT_RATE
        mode = settings.MODE
        if mode == "PROD" and request.path == "/api/graphql":
            rate_limited = is_ratelimited(
                request=request,
                group=group,
                key=key,
                rate=rate,
                method=is_ratelimited.ALL,
                increment=True,
            )
            if rate_limited:
                return JsonResponse({"detail": "Rate limit exceeded"}, status=429)
        response = self.get_response(request)
        return response


class AdminLogoutMiddleware:
    """
    Middleware to clear all user sessions when they log out from Django Admin.
    """

    LOGOUT_URL = f"/{settings.SITE_ROOT()}admin/logout/"

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)

        if request.path.startswith(self.LOGOUT_URL):
            response.delete_cookie("JWT")
            logger.info("Cleared all sessions after admin panel logout")

        return response



# Generated by Django 3.2.19 on 2023-06-01 11:45
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    psql_code = "select 1"
    mssql_code = "ALTER TABLE [tblOfficer] ALTER COLUMN [JsonExt]"

    dependencies = [
        ("core", "0022_auto_20230516_1742"),
    ]

    operations = [
        migrations.RunSQL(
            f"{mssql_code} NVARCHAR(MAX)" if settings.MSSQL else psql_code,
            reverse_sql=f"{mssql_code} TEXT" if settings.MSSQL else psql_code,
        ),
    ]



from datetime import timedelta, date, datetime

__all__ = ["is_midnight", "datetimedelta"]


def _cmperror(x, y):
    raise TypeError("can't compare '%s' to '%s'" % (type(x).__name__, type(y).__name__))


def is_midnight(dt):
    if hasattr(dt, "hour") and dt.hour != 0:
        return False
    if hasattr(dt, "minute") and dt.minute != 0:
        return False
    if hasattr(dt, "second") and dt.second != 0:
        return False
    if hasattr(dt, "microsecond") and dt.microsecond != 0:
        return False
    return True


def _cmp(x, y):
    return 0 if x == y else 1 if x > y else -1


def _add_month(dt):
    from core import calendar

    return dt + timedelta(calendar.monthdayscount(dt.year, dt.month))


def _sub_month(dt):
    from core import calendar

    if dt.month == 1:
        prev_year = dt.year - 1
        return dt - timedelta(
            calendar.monthdayscount(prev_year, calendar.yearmonthscount(prev_year))
        )
    else:
        return dt - timedelta(calendar.monthdayscount(dt.year, dt.month - 1))


class datetimedelta(object):
    __slots__ = ["_years", "_months", "_timedelta", "_hashcode"]

    def __init__(
        self,
        years=0,
        months=0,
        weeks=0,
        days=0,
        hours=0,
        minutes=0,
        seconds=0,
        microseconds=0,
    ):
        self._years = years
        self._months = months
        self._timedelta = timedelta(
            weeks=weeks,
            days=days,
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            microseconds=microseconds,
        )
        self._hashcode = -1

    @property
    def years(self):
        return self._years

    @property
    def months(self):
        return self._months

    @property
    def days(self):
        return self._timedelta.days

    @property
    def seconds(self):
        return self._timedelta.seconds

    @property
    def microseconds(self):
        return self._timedelta.microseconds

    @classmethod
    def from_timedelta(cls, td):
        return datetimedelta(
            years=0,
            months=0,
            days=td.days,
            seconds=td.seconds,
            microseconds=td.microseconds,
        )

    def _add_years(self, dt):
        while True:  # there are many '29/02' in nepali calendar
            try:
                return dt.replace(year=dt.year + self._years)
            except ValueError:
                return (dt - datetimedelta(days=1)) + self

    def _add_months(self, dt):
        for i in range(abs(self._months)):
            if self._months > 0:
                dt = _add_month(dt)
            else:
                dt = _sub_month(dt)
        return dt

    def add_to_date(self, dt):
        dt = self._add_years(dt)
        dt = self._add_months(dt)
        return dt + self._timedelta

    def add_to_datetime(self, datetime):
        return self.add_to_date(datetime)

    def __add__(self, other):
        if isinstance(other, (datetimedelta)):
            return datetimedelta(
                years=self.years + other.years,
                months=self.months + other.months,
                days=self.days + other.days,
                seconds=self.seconds + other.seconds,
                microseconds=self.microseconds + other.microseconds,
            )
        elif isinstance(other, (date, datetime)):
            return self.add_to_date(other)
        return NotImplemented

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        if isinstance(other, (datetimedelta)):
            return datetimedelta(
                years=self.years - other.years,
                months=self.months - other.months,
                days=self.days - other.days,
                seconds=self.seconds - other.seconds,
                microseconds=self.microseconds - other.microseconds,
            )
        return NotImplemented

    def __rsub__(self, other):
        if isinstance(other, (date, datetime)):
            sub = -self
            return sub.__add__(other)
        return self.__sub__(other)

    def __neg__(self):
        return datetimedelta(
            years=-self.years,
            months=-self.months,
            days=-self.days,
            seconds=-self.seconds,
            microseconds=-self.microseconds,
        )

    def __pos__(self):
        return self

    def __abs__(self):
        if self.years < 0:
            return -self
        if self.years == 0 and self.months < 0:
            return -self
        if self.years == 0 and self.months == 0:
            td = abs(self._timedelta)
            return datetimedelta(
                years=0,
                months=0,
                days=td.days,
                seconds=td.seconds,
                microseconds=td.microseconds,
            )
        return self

    def __mul__(self, other):
        if isinstance(other, int):
            return datetimedelta(
                years=self.years * other,
                months=self.months * other,
                days=self.days * other,
                seconds=self.seconds * other,
                microseconds=self.microseconds * other,
            )
        return NotImplemented

    __rmul__ = __mul__

    def __eq__(self, other):
        if isinstance(other, timedelta):
            return self._cmp(datetimedelta.from_timedelta(other)) == 0
        if isinstance(other, datetimedelta):
            return self._cmp(other) == 0
        else:
            _cmperror(self, other)

    def __le__(self, other):
        if isinstance(other, datetimedelta):
            return self._cmp(other) <= 0
        else:
            _cmperror(self, other)

    def __lt__(self, other):
        if isinstance(other, datetimedelta):
            return self._cmp(other) < 0
        else:
            _cmperror(self, other)

    def __ge__(self, other):
        if isinstance(other, datetimedelta):
            return self._cmp(other) >= 0
        else:
            _cmperror(self, other)

    def __gt__(self, other):
        if isinstance(other, datetimedelta):
            return self._cmp(other) > 0
        else:
            _cmperror(self, other)

    def _cmp(self, other):
        assert isinstance(other, datetimedelta)
        return _cmp(self._getstate(), other._getstate())

    def __hash__(self):
        if self._hashcode == -1:
            self._hashcode = hash(self._getstate())
        return self._hashcode

    def __bool__(self):
        return (
            self.years != 0
            or self.months != 0
            or self.days != 0
            or self.seconds != 0
            or self.microseconds != 0
        )

    def _getstate(self):
        return (self._years, self._months, self._timedelta)

    def __reduce__(self):
        return (self.__class__, self._getstate())

    def __repr__(self):
        args = []
        if self.years:
            args.append("years=%d" % self.years)
        if self.months:
            args.append("months=%d" % self.months)
        if self.days:
            args.append("days=%d" % self.days)
        if self.seconds:
            args.append("seconds=%d" % self.seconds)
        if self.microseconds:
            args.append("microseconds=%d" % self.microseconds)
        if not args:
            args.append("0")
        return "%s.%s(%s)" % (
            self.__class__.__module__,
            self.__class__.__qualname__,
            ", ".join(args),
        )



from django.test import TestCase
from core.security import ObjectPermissions
from core.models import ModuleConfiguration


class ObjectPermissionsTest(TestCase):
    def test_perms_map(self):
        perms = ObjectPermissions()
        self.assertEquals(
            ["core.view_moduleconfiguration"],
            perms.get_required_object_permissions("GET", ModuleConfiguration),
        )
        self.assertEquals(
            ["core.add_moduleconfiguration"],
            perms.get_required_object_permissions("POST", ModuleConfiguration),
        )
        self.assertEquals(
            ["core.change_moduleconfiguration"],
            perms.get_required_object_permissions("PUT", ModuleConfiguration),
        )
        self.assertEquals(
            ["core.change_moduleconfiguration"],
            perms.get_required_object_permissions("PATCH", ModuleConfiguration),
        )
        self.assertEquals(
            ["core.delete_moduleconfiguration"],
            perms.get_required_object_permissions("DELETE", ModuleConfiguration),
        )



# Generated by Django 3.0.14 on 2021-06-15 16:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0013_users_api"),
    ]

    operations = [
        migrations.AlterField(
            model_name="usermutation",
            name="core_user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="mutations",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0018_auto_20230318_1551"),
    ]

    operations = [
        migrations.RunSQL(
            "ALTER TABLE [tblOfficer] ADD [JsonExt] TEXT"
            if settings.MSSQL
            else 'ALTER TABLE "tblOfficer" ADD "JsonExt" jsonb'
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="openimis-be-core",
    version="1.10.0",
    packages=find_packages(),
    include_package_data=True,
    license="GNU AGPL v3",
    description="The openIMIS Backend Core reference module.",
    # long_description=README,
    url="https://openimis.org/",
    author="Xavier Gillmann",
    author_email="xgillmann@bluesquarehub.com",
    install_requires=[
        "isodate",
        "django",
        "django-db-signals",
        "djangorestframework",
        "cached-property",
        "nepalicalendar",
        "django-simple-history",
        "django-dirtyfields",
        "websocket-client",
        "jsonschema",
        "password-validator",
        "zxcvbn",
        "django-ratelimit",
        "pandas",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from core.utils import *  # noqa: F401,F403
from core.datetimes.shared import *  # noqa: F401,F403
from core.custom_lookups import *  # noqa: F401,F403

default_app_config = "core.apps.CoreConfig"

# For IDE support, filled at runtime
datetime = None



# Generated by Django 2.1.9 on 2019-07-26 07:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="moduleconfiguration",
            name="is_disabled_until",
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="moduleconfiguration",
            name="is_exposed",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="moduleconfiguration",
            name="layer",
            field=models.CharField(
                choices=[("fe", "frontend"), ("be", "backend")],
                default="be",
                max_length=2,
            ),
        ),
    ]
